package builtin;

import env.*;
import runEnv.*;
import runEnv.basicValue.*;
import text.*;

public class BuiltIn {
	public static void eval( String name, RunEnv runEnv ) throws ReturnException {
		PtrValue locals = runEnv.locals();
		if ( name.equals( "vsprintf" ) ) {
			PtrValue dest = locals.elementAt( 0 ).getValue().addressValue();
			PtrValue format = locals.elementAt( 1 ).getValue().addressValue();
			PtrValue args = locals.elementAt( 2 ).getValue().addressValue();
			vsPrintf( dest, format, args );
			throw new ReturnException( new VoidValue() );
			}
		else if ( name.equals( "print" ) ) {
			String s = locals.elementAt( 0 ).getValue().addressValue().stringValue();
			Print.interp().print( s );
			throw new ReturnException( new VoidValue() );
			}
		else if ( name.equals( "exit" ) ) {
			int value = locals.elementAt( 0 ).getValue().intValue();
			throw new ExitException( value );
			}
		throw new Error( "Unknown builtin " + name );
		}

	private final static int LEFT		=	-1;
	private final static int UNKNOWN	=	0;
	private final static int RIGHT		=	+1;

	private static void vsPrintf( PtrValue dest, PtrValue format, PtrValue params ) {
		long defaultAlign;
		long alignment;
		char padChar;
		String result;
		char c = format.getValue().charValue();
		while ( true ) {
			if ( c == 0 )
				break;
			if ( c == '%' ) {
				format = format.elementAt( 1 );
				c = format.getValue().charValue();
				if ( c == '+' ) {
					alignment = RIGHT;
					format = format.elementAt( 1 );
					c = format.getValue().charValue();
					}
				else if ( c == '-' ) {
					alignment = LEFT;
					format = format.elementAt( 1 );
					c = format.getValue().charValue();
					}
				else
					alignment = UNKNOWN;
				if ( c =='0' ) {
					padChar = '0';
					format = format.elementAt( 1 );
					c = format.getValue().charValue();
					}
				else
					padChar = ' ';
				int fieldWidth = 0;
				while ( '0' <= c && c <= '9' ) {
					fieldWidth = fieldWidth * 10 + ( c - '0' );
					format = format.elementAt( 1 );
					c = format.getValue().charValue();
					}
				switch ( c ) {
					case 'b':
						{
						long paramValue = params.getValue().intValue();
						params = params.elementAt( 1 );
						result = Long.toBinaryString( paramValue );
						defaultAlign = RIGHT;
						}
						break;
					case 'o':
						{
						long paramValue = params.getValue().intValue();
						params = params.elementAt( 1 );
						result = Long.toOctalString( paramValue );
						defaultAlign = RIGHT;
						}
						break;
					case 'd':
						{
						long paramValue = params.getValue().intValue();
						params = params.elementAt( 1 );
						result = Long.toString( paramValue );
						defaultAlign = RIGHT;
						}
						break;
					case 'x':
						{
						long paramValue = params.getValue().intValue();
						params = params.elementAt( 1 );
						result = Long.toHexString( paramValue );
						defaultAlign = RIGHT;
						}
						break;
					case 'c':
						{
						long paramValue = params.getValue().charValue();
						params = params.elementAt( 1 );
						result = "" + ( char ) paramValue;
						defaultAlign = LEFT;
						}
						break;
					case 's':
						{
						PtrValue paramValue = params.getValue().addressValue();
						params = params.elementAt( 1 );
						result = paramValue.stringValue();
						defaultAlign = LEFT;
						}
						break;
					default:
						result = "" + ( char ) c;
						defaultAlign = LEFT;
						break;
					}
				format = format.elementAt( 1 );
				c = format.getValue().charValue();
				if ( alignment == UNKNOWN )
					alignment = defaultAlign;
				if ( alignment == RIGHT )
					dest = dest.setStringValue( 
						padLeft( result, padChar, fieldWidth ) );
				else
					dest = dest.setStringValue( 
						padRight( result, padChar, fieldWidth ) );
				}
			else {
				dest = dest.setStringValue( "" + ( char ) c );
				format = format.elementAt( 1 );
				c = format.getValue().charValue();
				}
			}
		}
		
	private static String padLeft( String s, char padChar, int fieldWidth ) {
		while ( s.length() < fieldWidth )
			s = padChar + s;
		return s;
		}
		
	private static String padRight( String s, char padChar, int fieldWidth ) {
		while ( s.length() < fieldWidth )
			s = s + padChar;
		return s;
		}
		
	}
