package code;
import env.*;
//import runEnv.*;
import type.*;
import code.*;
import type.basicType.*;
import text.*;
import java.io.*;

public abstract class Code {

	public static void enter() {
		Print.compile().println( "{%+" );
		}

	public static void exit() {
		Print.compile().println( "%-}" );
		}

	public static void enter( String segment ) {
		Print.compile().println( segment + " {%+" );
		}

	public static void exit(  String segment ) {
		Print.compile().println( "%-} " + segment );
		}

	public static void enterBlock( String name, String extending, String using ) {
		String result = "public block " + name;
		if ( extending != null )
			result += " extends " + extending;
		if ( using != null )
			result += " uses " + using;
		result += " {%+";
		Print.compile().println( result );
		}

	public static void exitBlock(  String name ) {
		Print.compile().println( "%-} block " + name );
		}

	public static void comment(  String text ) {
		Print.compile().println( "// " + text );
		}

	public static void instrn( String opCode ) {
		if ( opCode == null )
			throw new Error( "null opCode" );
		Print.compile().println( opCode + ";" );
		}

	public static void instrn( String opCode, Address opd1 ) {
		if ( opCode == null )
			throw new Error( "null opCode" );
		Print.compile().println( opCode + " " + opd1 + ";" );
		}

	public static void instrn( String opCode, Address opd1, Address opd2 ) {
		if ( opCode == null )
			throw new Error( "null opCode" );
		Print.compile().println( opCode + " " + opd1 + ", " + opd2 + ";" );
		}

	public static void instrn( String opCode, Address opd1, Address opd2, Address opd3 ) {
		if ( opCode == null )
			throw new Error( "null opCode" );
		Print.compile().println( opCode + " " + opd1 + ", " + opd2 + ", " + opd3 + ";" );
		}

	public static void load( Type type, Register register, Address memory ) {
		if ( type instanceof CharType )
			loadB( register, memory );
		else if ( type instanceof ArrayType )
			loadA( register, memory );
		else if ( type instanceof ClassInstanceType )
			loadA( register, memory );
		else
			loadQ( register, memory );
		}

	public static void loadQ( Register register, Address memory ) {
		instrn( "ldq", register, memory );
		}

	public static void loadB( Register register, Address memory ) {
		instrn( "ldbu", register, memory );
		}

	public static void loadA( Register register, Address memory ) {
		instrn( "lda", register, memory );
		}

	public static void loadImm( Type type, Register register, Address value ) {
		if ( type instanceof FloatType )
			instrn( "ldit", register, value );
		else
			instrn( "ldiq", register, value );
		}

	public static void loadImm( Type type, Register register, String value ) {
		loadImm( type, register, new Literal( value ) );
		}

	public static void loadImm( Type type, Register register, long value ) {
		loadImm( type, register, new Literal( value ) );
		}

	public static void store( Type type, Register register, Address memory ) {
		if ( type instanceof CharType )
			storeB( register, memory );
		else
			storeQ( register, memory );
		}

	public static void storeB( Register register, Address memory ) {
		instrn( "stb", register, memory );
		}

	public static void storeQ( Register register, Address memory ) {
		instrn( "stq", register, memory );
		}

	public static void move( Register source, Register dest ) {
		instrn( "mov", source, dest );
		}

	public static void scaledAdd( Type type, Address base, Address index, Address result ) {
		int compileSize;
		if ( type instanceof PtrType ) {
			PtrType ptrType = (PtrType ) type;
			compileSize = ptrType.subType().size().compile();
			}
		else
			compileSize = 1;

		if ( compileSize == 1 )
			instrn( "addq", base, index, result );
		else if ( compileSize == 8 )
			instrn( "s8addq", index, base, result );
		else {
			loadImm( IntType.type, SpecialReg.assemTemp, compileSize );
			instrn( "mulq", index, SpecialReg.assemTemp, SpecialReg.assemTemp );
			instrn( "addq", base, SpecialReg.assemTemp, result );
			}
		}
		
	public static void scaledSub( Type type, Address base, Address index, Address result ) {
		int compileSize;
		if ( type instanceof PtrType ) {
			PtrType ptrType = (PtrType ) type;
			compileSize = ptrType.subType().size().compile();
			}
		else
			compileSize = 1;

		if ( compileSize == 1 )
			instrn( "subq", base, index, result );
		else {
			loadImm( IntType.type, SpecialReg.assemTemp, new Literal( compileSize ) );
			instrn( "mulq", index, SpecialReg.assemTemp, SpecialReg.assemTemp );
			instrn( "subq", base, SpecialReg.assemTemp, result );
			}
		}
		
	public static void inc( Type type, Address base, Register result ) {
		int compileSize;
		if ( type instanceof PtrType ) {
			PtrType ptrType = (PtrType ) type;
			compileSize = ptrType.subType().size().compile();
			}
		else
			compileSize = 1;
		
		if ( compileSize < 256 )
			instrn( "addq", base, new Literal( compileSize ), result );
		else
			loadA( result, new Displacement( compileSize, base ) );
		}
		
	public static void dec( Type type, Address base, Register result ) {
		int compileSize;
		if ( type instanceof PtrType ) {
			PtrType ptrType = (PtrType ) type;
			compileSize = ptrType.subType().size().compile();
			}
		else
			compileSize = 1;
		
		if ( compileSize < 256 )
			instrn( "subq", base, new Literal( compileSize ), result );
		else
			loadA( result, new Displacement( -compileSize, base ) );
		}
		
	public static void jsr() {
		instrn( "jsr" );
		}

	public static void jsr( Address address ) {
		instrn( "ldiq", SpecialReg.procValue, address );
		jsr();
		}

	public static void bsr( Relative address ) {
		instrn( "bsr", address );
		}

	public static void br( Relative address ) {
		instrn( "br", address );
		}

	public static void clear( Register register ) {
		instrn( "clr", register );
		}

	public static void ret() {
		instrn( "ret" );
		}

	public static void quad( String data ) {
		instrn( "quad", new Literal( data ) );
		}

	public static void quad( int data ) {
		instrn( "quad", new Literal( data ) );
		}

	public static void space( int compileSize ) {
		if ( compileSize == 0 )
			;
		else if ( compileSize == 1 )
			instrn( "byte" );
		else if ( compileSize == Type.WORD )
			instrn( "quad" );
		else
			instrn( "space", new Literal( compileSize ) );
		}

	public static void asciiz( String data ) {
		instrn( "asciiz", new Literal( "\"" + Convert.toFormatString( data ) + "\"" ) );
		instrn( "align" );
		}

	public static void align() {
		instrn( "align" );
		}

	public static void importFile( String fileName ) {
		instrn( "import", new Literal( "\"" + fileName + "\"" ) );
		}

	public static void labelDefn( String label ) {
		Print.compile().println( "%-" + label + ":%+" );
		}

	public static void labelDefn( Relative label ) {
		Print.compile().println( "%-" + label + ":%+" );
		}

	public static void publicLabelDefn( String label ) {
		Print.compile().println( "%-public " + label + ":%+" );
		}

	public static void publicLabelDefn( Relative label ) {
		Print.compile().println( "%-public " + label + ":%+" );
		}

	}
