package grammar;

import java.io.*;
import java.util.*;
import java_cup.runtime.*;
import text.*;
import env.*;

%%

%public
%type		Symbol
%char

%{

	public Symbol token( int tokenType, Object value ) {
		Print.error().debugln( "Obtain token " 
			+ sym.terminal_name( tokenType ) + " \"" + yytext() + "\"" );
		return new Symbol( tokenType, yychar, yychar + yytext().length(), value );
		}
		
	public Symbol token( int tokenType ) {
		return token( tokenType, yytext() );
		}
		
	private int lineNumber = 1;
	
	public int lineNumber() { return lineNumber; }
	
%}

%init{
	yybegin( NORMALSTATE );
%init}

intconst	=	[0-9]+
octDigit	=	[0-7]
hexDigit	=	[0-9a-fA-F]
escchar		=	\\([ntbrfva\\\'\"\?]|{octDigit}+|[xX]{hexDigit}+)
schar		=	[^\'\"\\\r\n]|{escchar}
charconst	=	\'{schar}\'
stringconst	=	\"{schar}*\"
ident		=	[A-Za-z][A-Za-z0-9]*
space		=	[\ \t]
newline		=	\r|\n|\r\n
%state NORMALSTATE COMMENT1STATE COMMENT2STATE LEXERRORSTATE

%%
<NORMALSTATE> {
	{newline}	{ lineNumber++; }
	{space}		{ }
	
	if			{ return token( sym.IF ); }
	then		{ return token( sym.THEN ); }
	else		{ return token( sym.ELSE ); }
	elif		{ return token( sym.ELIF ); }
	begin		{ return token( sym.BEGIN ); }
	end			{ return token( sym.END ); }
	while		{ return token( sym.WHILE ); }
	do			{ return token( sym.DO ); }
	for			{ return token( sym.FOR ); }
	return		{ return token( sym.RETURN ); }
	null		{ return token( sym.NULL ); }
	int			{ return token( sym.INT ); }
	char		{ return token( sym.CHAR ); }
	bool		{ return token( sym.BOOL ); }
	void		{ return token( sym.VOID ); }
	class		{ return token( sym.CLASS ); }
	extends		{ return token( sym.EXTENDS ); }
	this		{ return token( sym.THIS ); }
	
	";"			{ return token( sym.SEMICOLON ); }
	","			{ return token( sym.COMMA ); }
	"..."		{ return token( sym.ETC ); }
	"="			{ return token( sym.ASSIGN ); }
	
	"||"		{ return token( sym.OR ); }
	"&&"		{ return token( sym.AND ); }
	"!"			{ return token( sym.NOT ); }
	"<"			{ return token( sym.LT ); }
	">"			{ return token( sym.GT ); }
	"<="		{ return token( sym.LE ); }
	">="		{ return token( sym.GE ); }
	"=="		{ return token( sym.EQ ); }
	"!="		{ return token( sym.NE ); }
	"+"			{ return token( sym.PLUS ); }
	"-"			{ return token( sym.MINUS ); }
	"*"			{ return token( sym.TIMES ); }
	"/"			{ return token( sym.DIVIDE ); }
	"%"			{ return token( sym.MOD ); }
	"++"		{ return token( sym.INC ); }
	"--"		{ return token( sym.DEC ); }
	"&"			{ return token( sym.AMPERSAND ); }
	"^"			{ return token( sym.PTR ); }
	"("			{ return token( sym.LEFT ); }
	")"			{ return token( sym.RIGHT ); }
	"["			{ return token( sym.LEFTSQ ); }
	"]"			{ return token( sym.RIGHTSQ ); }
	"."			{ return token( sym.DOT ); }
	
	{intconst}	{ return token( sym.INTVALUE ); }
	{charconst}	{ return token( sym.CHARVALUE ); }
	{stringconst}	{ return token( sym.STRINGVALUE ); }
	{ident}		{ return token( sym.IDENT ); }
	"//"		{ yybegin( COMMENT1STATE ); }
	"/*"		{ yybegin( COMMENT2STATE ); }
	.			{
				yybegin( LEXERRORSTATE ); 
				return token( sym.error ); 
				}
	}
	
<LEXERRORSTATE> {
	";"			{
				yybegin( NORMALSTATE ); 
				return token( sym.SEMICOLON ); 
				}
	{newline}	{ lineNumber++; }
	.			{ }
	}

<COMMENT1STATE> {
	{newline}	{ lineNumber++; yybegin( NORMALSTATE ); }
	.			{ }
	}
	
<COMMENT2STATE> {
	"*/"		{ yybegin( NORMALSTATE ); }
	{newline}	{ lineNumber++; }
	.			{ }
	}

<<EOF>>			{ return token( sym.EOF ); }

