package node;
import grammar.*;
import env.*;
import type.*;
import type.basicType.*;
import code.*;
import text.*;
import runEnv.*;
import java.io.*;

public class ProgramNode extends Node {

	private DeclStmtListNode globalLibrary;
	private DeclStmtListNode declStmtList;
	private Decl main;
	
	public ProgramNode( DeclStmtListNode globalLibrary, DeclStmtListNode declStmtList ) {
		this.globalLibrary = globalLibrary;
		this.declStmtList = declStmtList;
		}

	public String toString() {
		return declStmtList + "%n";
		}
		
	public void genEnv() {
		super.genEnv( new Env( Env.ENV_GLOBAL ) );
		main = new Decl( "main", Decl.DECL_NONE, null, env() );
		env().setDecl( main );
		globalLibrary.genEnv( Decl.DECL_VAR, env() );
		declStmtList.genEnv( Decl.DECL_VAR, env() );
		}

	public void setType() {
		main.setType( new MethodType( VoidType.type, new DeclList() ) );
		globalLibrary.setType();
		declStmtList.setType();
		}

	public void checkType() {
		globalLibrary.checkType();
		declStmtList.checkType();
		}

	public void genOffset() {
		main.genOffset();
//		env().varOffset().set( 1, Type.WORD );
		globalLibrary.genOffset();
		declStmtList.genOffset();
//		env().createMethodTable();
		}

	public void eval() throws ReturnException {
		Multiple globalVars = new Multiple( "globals", env().varOffset().interp() );
//		DeclList globalMethods = env().methodTable();
//		globalVars.setValue( 0, new MethodTableValue( globalMethods ) );
		RunEnv runEnv = new RunEnv( new PtrValue( globalVars ), null, null );
		globalLibrary.eval( runEnv );
		declStmtList.eval( runEnv );
		}

	public void genGlobalCode() {
		Code.comment( "Initialisation code for globals" );
		Code.enterBlock( "global", null, null );
		Code.enterBlock( "init", "proc.sav0", "proc" );

		Code.enter( "code" );
		Code.publicLabelDefn( "enter" );
		Code.loadA( SpecialReg.stackPtr, new Displacement( "-max", SpecialReg.stackPtr ) );
		Code.storeQ( SpecialReg.retAddr, new Displacement( "savRet", SpecialReg.stackPtr ) );
		Code.move( SpecialReg.stackPtr, SpecialReg.framePtr );
		globalLibrary.genCode();
		declStmtList.genCode();
		Code.loadQ( SpecialReg.retAddr, new Displacement( "savRet", SpecialReg.stackPtr ) );
		Code.loadA( SpecialReg.stackPtr, new Displacement( "+max", SpecialReg.stackPtr ) );
		Code.ret();
		Code.exit( "code" );

		Code.enter( "local" );
		Code.publicLabelDefn( "max" );
		Code.exit( "local" );

		Code.exitBlock( "init" );
		Code.exitBlock( "global" );
		}
	
	public void genCode() {
		Code.instrn( "entry", new Literal( "main.enter" ) );
		Code.importFile( "../../IMPORT/register.h" );
		Code.importFile( "../../IMPORT/callsys.h" );
		Code.importFile( "../../IMPORT/proc.h" );
		Code.importFile( "../../IMPORT/invoc.h" );
		Code.importFile( "../../IMPORT/callsys.lib.s" );
		Code.importFile( "../../IMPORT/string.lib.s" );
		Code.importFile( "../../IMPORT/number.lib.s" );
		Code.importFile( "../../IMPORT/bool.lib.s" );
		Code.importFile( "../../IMPORT/io.lib.s" );
		Code.importFile( "../../LIBRARY/builtin.lib.s" );

		Code.enterBlock( "main", null, "register" );

		env().genGlobalDeclCode();
		env().genClassDeclCode();
		env().genMethodDeclCode();

		genGlobalCode();
		
		Code.enter( "code" );
		Code.publicLabelDefn( "enter" );
		Code.clear( SpecialReg.instPtr );
		Code.clear( SpecialReg.newInstPtr );

		Code.bsr( new Relative( "global.init.enter" ) );

		Code.clear( new IntArgReg( 0 ) );
		Code.bsr( new Relative( "Sys.exit.enter" ) );
		Code.exit( "code" );

		Code.exitBlock( "main" );
		}
	}
