package node.declNode;
import node.*;
import node.typeNode.*;
import env.*;
import type.*;
import code.*;
import type.basicType.*;
import runEnv.*;
import text.*;

public class MethodDeclNode extends DeclStmtNode implements DeclNode {

	private TypeNode returnTypeNode;
	private String ident;
	private DeclStmtListNode formalParams;
	private DeclStmtListNode body;

	public String ident() { return ident; }
	public DeclStmtListNode formalParams() { return formalParams; }
	public DeclStmtListNode body() { return body; }

	private Env localEnv;
	private Type returnType;
	private Decl decl;

	public Env localEnv() { return localEnv; }
	public Type returnType() { return returnType; }
	public Decl decl() { return decl; }

	public MethodDeclNode( 
		TypeNode returnTypeNode,
		String ident,
		DeclStmtListNode formalParams,
		DeclStmtListNode body ) {
		this.returnTypeNode = returnTypeNode;
		this.ident = ident;
		this.formalParams = formalParams;
		this.body = body;
		}
	
	public String toString() {
		String formalText, bodyText;
		if ( formalParams.size() == 0 )
			formalText = "()";
		else
			formalText = "(%+%n" + formalParams() + " )%-";
		if ( body == null )
			bodyText = ";";
		else if ( body.size() == 0 )
			bodyText = "%+%nbegin%nend%-";
		else
			bodyText = "%+%nbegin%+%n" + body + "%-%nend%-";
		return returnTypeNode + " " + ident 
			+ formalText + bodyText;
		}
		
	public void genEnv( int section, Env env ) {
		super.genEnv( section, env );
		localEnv = new Env( Env.ENV_METHOD, env );
		decl = env().declare( ident, Decl.DECL_METHOD, this );
		localEnv().setDecl( decl );
		returnTypeNode.genEnv( env );
		formalParams.genEnv( Decl.DECL_PARAM, localEnv );
		if ( body != null )
			body.genEnv( Decl.DECL_VAR, localEnv );
		}

	public void setType() {
		returnType = returnTypeNode.getType();
		formalParams().setType();
		if ( body != null )
			body().setType();
		DeclList paramDecls = localEnv().paramDecls();
		MethodType methodType = new MethodType( returnType, paramDecls );
		decl.setType( methodType );
		}

	public void checkType() {
		if ( body != null )
			body.checkType();
		}

	public void genOffset() {
		decl.genOffset();
		localEnv.varOffset().set( 0, 0 );
		if ( body != null )
			body.genOffset();
		localEnv.paramStart().set( 
			localEnv.varOffset().interp(), 
			localEnv.varOffset().compile() );
		formalParams.genOffset();
		}

	public void eval( RunEnv runEnv ) {
		}

	public void genDeclCode() {
		if ( body != null ) {
			Code.enterBlock( decl.declName(), "proc.sav0", "proc" );
			Code.enter( "code" );
			Code.align();
			Code.publicLabelDefn( "enter" );
			Code.loadA( SpecialReg.stackPtr, 
				new Displacement( "-params", SpecialReg.stackPtr ) );
			Code.storeQ( SpecialReg.retAddr, 
				new Displacement( "savRet", SpecialReg.stackPtr ) );
			Code.storeQ( SpecialReg.framePtr, 
				new Displacement( "savFP", SpecialReg.stackPtr ) );
			Code.storeQ( SpecialReg.instPtr, 
				new Displacement( "savIP", SpecialReg.stackPtr ) );
			Code.move( SpecialReg.newInstPtr, SpecialReg.instPtr );
			Code.move( SpecialReg.stackPtr, SpecialReg.framePtr );
			body.genCode();
			Code.clear( new IntTempReg( 0 ) );
			Code.labelDefn( "return" );
//			Code.move( SpecialReg.instPtr, SpecialReg.newInstPtr );
			Code.loadQ( SpecialReg.instPtr, 
				new Displacement( "savIP", SpecialReg.stackPtr ) );
			Code.loadQ( SpecialReg.framePtr, 
				new Displacement( "savFP", SpecialReg.stackPtr ) );
			Code.loadQ( SpecialReg.retAddr, 
				new Displacement( "savRet", SpecialReg.stackPtr ) );
			Code.loadA( SpecialReg.stackPtr, 
				new Displacement( "+params", SpecialReg.stackPtr ) );
			Code.ret();
			Code.exit( "code" );
			localEnv.genLocalDecls();
			Code.exitBlock( decl.declName() );
			}
		}

	public void genCode() {
		}

	}
