package node.exprNode;
import node.*;
import node.exprNode.variableNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;
import text.*;
import java.util.*;

public class ExprListNode extends ListNode< ExprNode > {

	public ExprListNode() {
		super( ", " );
		}
	
	public ExprListNode( ExprNode node ) {
		this();
		addElement( node );
		}
		
	public void genEnv( Env env ) {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.genEnv( env );
			}
		}
		
	public Type checkType() {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.checkType();
			}
		return null;
		}
	
	public void castParams( MethodType methodType ) {
		Offset paramOffset = new Offset();
		int formalSize = methodType.formalDecls().size();
		int actualSize = size();

		if ( actualSize > 0 && formalSize == 0 )
			throw new Error( 
				"Method has actual parameters, but no formal parameters" );
		else if ( formalSize > 0 ) {
			Decl lastDecl = 
				methodType.formalDecls().elementAt( formalSize - 1 );
			if ( lastDecl.ident().equals( "..." ) ) {
				--formalSize;
				if ( actualSize < formalSize )
					throw new Error( "Too few actual parameters" );
				}
			else if ( actualSize < formalSize )
				throw new Error( "Too few actual parameters" );
			else if ( actualSize > formalSize )
				throw new Error( "Too many actual parameters" );
			}
		for ( int i = 0; i < formalSize; i++ ) {
			ExprNode expr = elementAt( i );
			Decl formalDecl = methodType.formalDecls().elementAt( i );
			Type formalType = 
				methodType.formalDecls().elementAt( i ).type();
			setElementAt( i, expr.castTo( Type.CAST_PARAM, formalType ) );
			}
		}

	public void eval( RunEnv runEnv, PtrValue address ) {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			RunValue exprValue = expr.eval( runEnv );
			address.elementAt( i ).setValue( exprValue );
			Print.error().debugln( "param " + i + " = " + exprValue );
			}
		}
		
	public Register saveReg( int regType, Usage usage ) {
		return null;
		}
		
	public void evalCode( Node sibling ) {
		setSibling( sibling );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.evalCode( sibling );
			Register register = expr.loadReg( usage() );
			Code.storeQ( register, 
				new Displacement( "invoc.act" + i, SpecialReg.stackPtr ) );
			}
		}
	}
