package node.exprNode;
import node.*;
import node.exprNode.variableNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;
import text.*;

public class VarExprNode extends ExprNode {

	private VariableNode variable;
	private Type addressType;

	public VariableNode variable() { return variable; }

	public VarExprNode( VariableNode variable ) {
		this.variable = variable;
		precedence = PREC_PRIMARY;
		}

	public String toString() {
		return "" + variable;
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		variable.genEnv( env );
		}

	public Type checkType() {
		addressType = variable.checkType();
		type = addressType.formalType();
		return type;
		}
		
	public RunValue eval( RunEnv runEnv ) {
		if ( addressType instanceof ArrayType 
			|| addressType instanceof ClassInstanceType )
			return variable.evalAddr( runEnv );
		else
			return variable.evalAddr( runEnv ).getValue();
		}

	public void evalCode( Node sibling ) {
		setSibling( sibling );
		variable.evalAddrCode( sibling );
		Address address = variable.loadAddress( usage() );
		if ( addressType instanceof ArrayType
			|| addressType instanceof ClassInstanceType ) {
			if ( address instanceof Indirect ) {
				Indirect indirectAddress = ( Indirect ) address;
				setAddress( indirectAddress.base() );
				}
			else {
				Register register = obtainReg( type, usage() );
				Code.loadA( register, address );
				setAddress( register );
				}
			}
		else
			setAddress( address );
		}
	}
