package node.exprNode.binaryNode.arithNode;
import node.*;
import node.exprNode.*;
import node.exprNode.binaryNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import code.*;

public class PlusNode extends ArithNode {

	public PlusNode( ExprNode left, ExprNode right ) {
		super( left, right );
		precedence = PREC_ADD;
		operator = "+";
		opCode = "addq";
		}

	public Type checkType() {
		Type leftType = left.checkType();
		Type rightType = right.checkType();
		if ( leftType instanceof PtrType ) {
			right = right.castTo( Type.CAST_IMPLICIT, IntType.type );
			type = leftType;
			return type;
			}
		else {
			operandType = IntType.type;
			left = left.castTo( Type.CAST_IMPLICIT, operandType );
			right = right.castTo( Type.CAST_IMPLICIT, operandType );
			type = operandType;
			return type;
			}
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		if ( type instanceof PtrType ) {
			PtrType ptrType = ( PtrType ) type;
			int size = ptrType.subType().size().interp();
			return leftValue.addressValue().elementAt( size * rightValue.intValue() );
			}
		else
			return new IntValue( leftValue.intValue() + rightValue.intValue() );
		}

	public void evalCode( Node sibling ) {
		super.evalCode( sibling );
		Code.scaledAdd( type, left.address(), right.address(), address() );
		}
	}
