package node.exprNode.binaryNode.relationNode;
import node.*;
import node.exprNode.*;
import node.exprNode.binaryNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import code.*;

public class NotEqualNode extends RelationNode {

	public NotEqualNode( ExprNode left, ExprNode right ) {
		super( left, right );
		precedence = PREC_EQ;
		operator = "!=";
		opCode = "cmpeq";
		}

	public Type checkType() {
		Type leftType = left.checkType();
		Type rightType = right.checkType();
		operandType = Type.balanceType( leftType, rightType );
		if ( operandType == null )
			throw new Error( "Incompatible operand types of " + leftType + 
				" and " + rightType + " for " + operator );
		left = left.castTo( Type.CAST_IMPLICIT, operandType );
		right = right.castTo( Type.CAST_IMPLICIT, operandType );
		type = BoolType.type;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		return new BoolValue( ! leftValue.equals( rightValue ) );
		}
		
	public void evalCode( Node sibling ) {
		super.evalCode( sibling );
		Code.instrn( "xor", address(), new Literal( 1 ), address() );
		}
		
	public void boolCode(
		Node sibling,
		Relative trueLabel, 
		Relative falseLabel, 
		boolean flowIntoTrue ) {
		super.evalCode( sibling );
		if ( flowIntoTrue ) {
			Code.instrn( "blbs", address(), falseLabel );
			}
		else {
			Code.instrn( "blbc", address(), trueLabel );
			}
		}

	}
