package node.exprNode.methodNameNode;
import node.*;
import node.declNode.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import code.*;
import text.*;

public class MemberMethodNameNode extends MethodNameNode {

	private ExprNode object;
	private Type objectType;
	private String ident;

	public MemberMethodNameNode( ExprNode object, String ident ) {
		this.object = object;
		this.ident = ident;
		}

	public String toString() {
		return object + "." + ident;
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		object.genEnv( env );
		}

	public Type checkType() {
		Type oType = object.checkType();
		if ( ! ( oType instanceof PtrType ) )
			throw new Error( "Selection must be from a reference" );
		PtrType ptrType = ( PtrType ) oType;
		Type sType = ptrType.subType();
		if ( ! ( sType instanceof ClassInstanceType ) )
			throw new Error( 
				"Selection must be from a reference to an instance of class type" );
		ClassInstanceType instanceType = ( ClassInstanceType ) sType;
		decl = instanceType.env().searchExtended( ident );
		if ( decl == null )
			throw new Error( "Undeclared method " + ident );
		if ( decl.section() != Decl.DECL_METHOD )
			throw new Error( ident + " must be method" );
		type = ( MethodType ) decl.type();
		return type;
		}

	public ObjectMethodPair eval( RunEnv runEnv ) {
		switch ( decl.env().envKind() ) {
			case Env.ENV_CLASS:
				{
				PtrValue instanceValue = object.eval( runEnv ).addressValue();
				DeclList methodTable = 
					instanceValue.getValue().methodTable();
				return new ObjectMethodPair( instanceValue,
					methodTable.elementAt( decl.offset().interp() ) );
				}
			default:
				throw new Error( 
					"Invalid section of " + Decl.sectionText( decl.section() ) );
			}
		}

	public Register saveReg( int dataType, Usage usage ) {
		return null;
		}
		
	public void evalCode( Node sibling ) {
		setSibling( sibling );
		object.evalCode( sibling );
		Register register = object.loadReg( usage() );
		Code.move( register, SpecialReg.newInstPtr );
		Code.load( IntType.type, SpecialReg.assemTemp, 
			new Displacement( 
				decl.env().absoluteName() + ".field.methodTablePtr", 
				SpecialReg.newInstPtr ) );
		Code.load( IntType.type, SpecialReg.procValue, 
			new Displacement( decl.absoluteName(), SpecialReg.assemTemp ) );
		}

	}
