package node.exprNode.primaryNode;

import builtin.*;
import node.*;
import node.exprNode.*;
import node.exprNode.methodNameNode.*;
import node.declNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import code.*;
import text.*;

public class InvocationNode extends PrimaryNode {

	private MethodNameNode methodName;
	private ExprListNode actualParamList;
	private MethodType methodType;

	public InvocationNode( 
		MethodNameNode methodName, 
		ExprListNode actualParamList ) {
		this.methodName = methodName;
		this.actualParamList = actualParamList;
		}
	
	public String toString() {
		if ( actualParamList.size() == 0 )
			return methodName + "()";
		else
			return methodName + "( " + actualParamList + " )";
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		methodName.genEnv( env );
		actualParamList.genEnv( env );
		}

	public Type checkType() {
		methodType = ( MethodType ) methodName.checkType();
		actualParamList.checkType();
		actualParamList.castParams( methodType );
		type = methodType.returnType();
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		ObjectMethodPair objectMethodPair = methodName.eval( runEnv );
		PtrValue instance = objectMethodPair.instanceValue();
		Decl methodDecl = objectMethodPair.methodDecl();
		MethodDeclNode methodDeclNode = ( MethodDeclNode ) methodDecl.declNode();
		Env localEnv = methodDeclNode.localEnv();
		int frameSize = 
			localEnv.paramStart().interp() + actualParamList.size();
		Multiple locals = new Multiple( "" + methodName, frameSize );
		actualParamList.eval( runEnv, 
			new PtrValue( locals, frameSize - actualParamList.size() ) );
		try {
			RunEnv invocationRunEnv 
				= new RunEnv( runEnv.globals(), 
					instance, new PtrValue( locals ) );
			DeclStmtListNode body = methodDeclNode.body();
			if ( body == null )
				BuiltIn.eval( methodDeclNode.ident(), invocationRunEnv );
			else
				methodDeclNode.body().eval( invocationRunEnv );
			}
		catch ( ReturnException exception ) {
			return exception.returnValue();
			}
		return VoidValue.value;
		}

	public void evalCode( Node sibling ) {
		setSibling( sibling );
		saveAllReg( usage() );
		Code.loadA( SpecialReg.stackPtr, 
			new Displacement( "-invoc.act" + actualParamList.size(), 
				SpecialReg.stackPtr ) );
		actualParamList.evalCode( sibling );
		methodName.evalCode( sibling );
		Code.jsr();
		Code.loadA( SpecialReg.stackPtr, 
			new Displacement( "+invoc.act" + actualParamList.size(), 
				SpecialReg.stackPtr ) );
		setAddress( obtainReg( type, usage() ) );
		}
	}
