package node.exprNode.variableNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;

public class IndexVariableNode extends VariableNode {

	private ExprNode pointer;
	PtrType pointerType;
	private ExprNode index;
	private Register resultReg;
	
	public IndexVariableNode( ExprNode pointer, ExprNode index ) {
		this.pointer = pointer;
		this.index = index;
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		pointer.genEnv( env );
		index.genEnv( env );
		}

	public Type checkType() {
		Type aType = pointer.checkType();
		if ( ! ( aType instanceof PtrType ) )
			throw new Error( 
				"Attempting to index a non-pointer type " + aType );
		pointerType = ( PtrType ) aType;
		Type indexType = index.checkType();
		index = index.castTo( Type.CAST_IMPLICIT, IntType.type );
		type = pointerType.subType();
		return type;
		}

	public String toString() {
		return pointer.toString( ExprNode.PREC_PRIMARY ) + "[ " + index + " ]";
		}

	public PtrValue evalAddr( RunEnv runEnv ) {
		PtrValue pointerAddress = pointer.eval( runEnv ).addressValue();
		RunValue indexValue = index.eval( runEnv );
		return pointerAddress.elementAt( 
			type.size().interp() * indexValue.intValue() );
		}

	public void evalAddrCode( Node sibling ) {
		setSibling( sibling );
		pointer.evalCode( sibling );
		index.evalCode( pointer );
			pointer.loadReg( usage().plus( index.address() ) );
		index.loadReg( usage().plus( pointer.address() ) );
		resultReg = obtainReg( pointerType, usage() );
		Code.scaledAdd( 
			pointerType, pointer.address(), index.address(), resultReg );
		setAddress( new Indirect( resultReg ) );
		}
	}
