package node.exprNode.variableNode;
import node.*;
import node.exprNode.*;
import node.exprNode.prefixNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;
import text.*;

public class MemberVariableNode extends VariableNode {

	private ExprNode object;
	private String ident;
	private Decl decl;

	public MemberVariableNode( ExprNode object, String ident ) {
		this.object = object;
		this.ident = ident;
		}

	public String toString() {
		return object.toString( ExprNode.PREC_PRIMARY ) + "." + ident;
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		object.genEnv( env );
		}

	public Type checkType() {
		Type oType = object.checkType();
		if ( ! ( oType instanceof PtrType ) )
			throw new Error( "Selection must be from a reference" );
		PtrType ptrType = ( PtrType ) oType;
		Type sType = ptrType.subType();
		if ( ! ( sType instanceof ClassInstanceType ) )
			throw new Error( 
				"Selection must be from a reference to an instance of class type" );
		ClassInstanceType instanceType = ( ClassInstanceType ) sType;
		decl = instanceType.env().searchExtended( ident );
		if ( decl == null )
			throw new Error( "Undeclared field " + ident );
		if ( decl.section() != Decl.DECL_VAR )
			throw new Error( ident + " must be field" );
		type = decl.type();
		return type;
		}

	public PtrValue evalAddr( RunEnv runEnv ) {
		PtrValue address = 
			object.eval( runEnv ).addressValue().elementAt( decl.offset().interp() );
		Print.error().debugln( "Evaluate " + this 
			+ " offset " + decl.offset().interp()
			+ " to give address with contents " + address.getValue() );
		return address;
		}

	public void evalAddrCode( Node sibling ) {
		setSibling( sibling );
		object.evalCode( sibling );
		Register base = object.loadReg( usage() );
		setAddress( new Displacement( decl.absoluteName(), base ) );
		}
	}
