package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;

public class ForStmtNode extends StmtNode {

	private ExprNode initial;
	private ExprNode cond;
	private ExprNode increment;
	private DeclStmtListNode stmtList;

	public ForStmtNode( 
		ExprNode initial, 
		ExprNode cond, 
		ExprNode increment, 
		DeclStmtListNode stmtList ) {
		this.initial = initial;
		this.cond = cond;
		this.increment = increment;
		this.stmtList = stmtList;
		}

	public String toString() {
		return "for " + initial + "; " + cond + "; " 
			+ increment + " do%+%n" + stmtList + "%-%nend";
		}

	public void genEnv( int section, Env env ) {
		super.genEnv( section, env );
		initial.genEnv( env );
		cond.genEnv( env );
		increment.genEnv( env );
		stmtList.genEnv( section, env );
		}

	public void checkType() {
		Type initialType = initial.checkType();
		initial = initial.castTo( Type.CAST_IMPLICIT, VoidType.type );
		Type condType = cond.checkType();
		cond = cond.castTo( Type.CAST_IMPLICIT, BoolType.type );
		Type incrementType = increment.checkType();
		increment = increment.castTo( Type.CAST_IMPLICIT, VoidType.type );
		stmtList.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		for ( 
			initial.eval( runEnv ); 
			cond.eval( runEnv ).boolValue(); 
			increment.eval( runEnv ) )
			stmtList.eval( runEnv );
		}

	public void genCode() {
		Code.enter();
		Code.labelDefn( "for" );
		initial.evalCode( null );
		Code.labelDefn( "while" );
		cond.boolCode( null, 
			new Relative( "do" ), new Relative( "end" ), true );
		Code.labelDefn( "do" );
		stmtList.genCode();
		Code.labelDefn( "continue" );
		increment.evalCode( null );
		Code.br( new Relative( "while" ) );
		Code.labelDefn( "break" );
		Code.labelDefn( "end" );
		Code.exit();
		}
	}
