package type;

import env.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import code.*;
import node.*;
import node.declNode.*;

public  class ClassInstanceType extends Type {

	private String ident;
	private Env env;

	public String ident() { return ident; }
	public Env env() { return env; }

	public ClassInstanceType( String ident, Env env ) {
		this.ident = ident;
		this.env = env;
		}
	
	public boolean equals( Type type ) {
		if ( ! ( type instanceof ClassInstanceType ) )
			return false;
		ClassInstanceType classInstanceType = ( ClassInstanceType ) type;
		return classInstanceType.ident().equals( ident );
		}

	public boolean isCastable( int castKind, Type type ) {
		if ( type instanceof ClassInstanceType ) {
			ClassInstanceType instanceType = ( ClassInstanceType ) type;
			return env.isExtensionOf( instanceType.env() );
			}
		return false;
		}

	public String toString() {
		return "instance " + ident;
		}

	public Type formalType() {
		return new PtrType( this );
		}

	public Offset size() {
		return env.varOffset();
		}
		
	public void initAddress( RunEnv runEnv, PtrValue address ) 
		throws ReturnException {
		address.setValue( new MethodTableValue( env.methodTable() ) );
		RunEnv localRunEnv 
			= new RunEnv( runEnv.globals(), address, null );
		env.initInstance( localRunEnv );
		}

	public void initAddressCode() {
		Code.loadImm( IntType.type, SpecialReg.assemTemp, 
			env.absoluteName() + ".methodTable.enter" );
		Code.store( IntType.type, SpecialReg.assemTemp, 
			new Indirect( SpecialReg.newInstPtr ) );
		Code.bsr( new Relative( env.absoluteName() + ".initInstance.enter" ) );
		}
	}
