package node;

public class Node {
	Node[] child = new Node[ 4 ];
	String format;
	String text;

	public void print( int indent ) {
		int charPosn = 0;
		int childPosn = 0;
		// System.err.println( "print( " + indent + " )" );
		if ( format == null ) {
			// System.err.println( "Null format string" );
			return;
			}
		for ( charPosn = 0; charPosn < format.length(); charPosn++ ) {
			if ( format.charAt( charPosn ) == '%' ) {
				// System.err.println( "Match %" + format.charAt( charPosn + 1 ) );
				switch ( format.charAt( ++charPosn ) ) {
					case 't':
						// Print next child
						if ( child[ childPosn ] != null )
							child[ childPosn ].print( indent );
						childPosn++;
						break;
					case 's':
						// Print text string
						if ( text != null )
							Out.print( indent, text );
						break;
					case 'n':
						// Print a newline
						Out.newline();
						break;
					case '+':
						// Increment indenting level
						indent++;
						break;
					case '-':
						// decrement indenting level
						--indent;
						break;
					case '<':
						// Print a newline if next child is not null
						if ( child[ childPosn ] != null )
							Out.newline();
						break;
					case '>':
						// Print a newline if previous child is not null
						if ( child[ childPosn - 1 ] != null )
							Out.newline();
						break;
					case '(':
						// Print a blank if next child is not null
						if ( child[ childPosn ] != null )
							Out.print( indent, ' ' );
						break;
					case ')':
						// Print a blank if previous child is not null
						if ( child[ childPosn - 1 ] != null )
							Out.print( indent, ' ' );
						break;
					case '{':
						// Print { for compound statement on same line, 
						// but other substatements on a new line
						if ( child[ childPosn ] instanceof BlockNode ) {
							Out.print( indent, ' ' );
							indent++;
							}
						else {
							indent++;
							Out.newline();
							}
						break;
					case 'i':
						// Print if and compound statement on same line, 
						// but other substatements on a new line 
						// - used in if statements to generate "else if"
						if ( child[ childPosn ] instanceof IfNode
							|| child[ childPosn ] instanceof IfElseNode ) {
							Out.print( indent, ' ' );
							}
						else if ( child[ childPosn ] instanceof BlockNode ) {
							Out.print( indent, ' ' );
							indent++;
							}
						else {
							indent++;
							Out.newline();
							}
						break;
					case '%':
						Out.print( indent, '%' );
						break;
					default:
						System.err.println( "Invalid % string in print()" );
						return;
					}
				}
			else {
				Out.print( indent, format.charAt( charPosn ) );
				}
			}
		}
	
	public Node() {
		// System.err.println( "Node0" );
		}

	public Node( Node n0 ) {
		// System.err.println( "Node1" );
		child[ 0 ] = n0;
		}

	public Node( Node n0, Node n1 ) {
		// System.err.println( "Node2" );
		child[ 0 ] = n0;
		child[ 1 ] = n1;
		}

	public Node( Node n0, Node n1, Node n2 ) {
		// System.err.println( "Node3" );
		child[ 0 ] = n0;
		child[ 1 ] = n1;
		child[ 2 ] = n2;
		}

	public Node( Node n0, Node n1, Node n2, Node n3 ) {
		// System.err.println( "Node4" );
		child[ 0 ] = n0;
		child[ 1 ] = n1;
		child[ 2 ] = n2;
		child[ 3 ] = n3;
		}

	}
