package grammar;

import java.io.*;
import java_cup.runtime.*;
import text.*;

%%

%public
%type		Symbol
%char

%{
	public Symbol token( int tokenType ) {
		return new Symbol( tokenType, yychar, yychar + yytext().length(), "" );
		}

	public Symbol token( int tokenType, Object value ) {
		return new Symbol( tokenType, yychar, yychar + yytext().length(), value );
		}
%}

%init{
	yybegin( NORMAL );
%init}

escchar		=	\\[^0-3xX]|\\[0-3][0-7][0-7]|\\[xX][0-9a-fA-F][0-9a-fA-F]

cchar		=	[^\"\\\r\n\$\(\)\|\?\+\*\.\[\]]|{escchar}
ichar		=	[^\^\\\r\n\[\]\-]|{escchar}
schar		=	[^\"\\\r\n]|{escchar}

%state NORMAL INSIDESQ 

%%

<NORMAL> {
	"("					{ return token( sym.LEFT ); }
	")"					{ return token( sym.RIGHT ); }
	"|"					{ return token( sym.OR ); }
	"?"					{ return token( sym.QUEST ); }
	"+"					{ return token( sym.PLUS ); }
	"*"					{ return token( sym.STAR ); }
	"."					{ return token( sym.DOT ); }

	"["					{
						yybegin( INSIDESQ ); 
						return token( sym.LEFTSQ );
						}
	\"{schar}*\"		{ 
						String s = yytext();
						return token( sym.STRING, 
							Text.parseString( s.substring( 1, s.length() - 1 ) ) );
						}
	{cchar}				{
						String s = yytext();
						return token( sym.CHAR, Text.parseChar( s ) );
						}
	.|\n				{ return token( sym.error ); }
	}
	
<INSIDESQ> {
	"]"					{
						yybegin(  NORMAL );
						return token( sym.RIGHTSQ );
						}
	"^"					{ return token( sym.CARET ); }
	"-"					{ return token( sym.MINUS ); }
	{ichar}				{
						String s = yytext();
						return token( sym.CHAR, Text.parseChar( s ) );
						}
	.|\n				{ return token( sym.error ); }
	}

<<EOF>>					{ return token( sym.EOF ); }

