package grammar;

import text.*;
import java.io.*;
import java_cup.runtime.*;

%%

%public
%type		Symbol
%char

%{
	private int lineNumber = 1;
	public int lineNumber() { return lineNumber; }
	
	public Symbol token( int tokenType ) {
		System.err.println( "Obtain token " 
			+ sym.terminal_name( tokenType ) + " \"" + yytext() + "\"" );
		return new Symbol( tokenType, yychar, 
			yychar + yytext().length(), yytext() );
		}
%}

%init{
	yybegin( NORMAL );
%init}

number		=	[0-9]+
octDigit	=	[0-7]
hexDigit	=	[0-9a-fA-F]
escchar		=	\\([ntbrfva\\\'\"\?]|{octDigit}+|[xX]{hexDigit}+)
schar		=	[^\"\\\r\n]|{escchar}
string		=	\"{schar}*\"
ident		=	[A-Za-z][A-Za-z0-9]*
space		=	[\ \t]
newline		=	\r|\n|\r\n
%state NORMAL LEXERROR

%%

<NORMAL> {
	{newline}	{ lineNumber++; }
	{space}		{ }

	print		{ return token( sym.PRINT ); }
	if			{ return token( sym.IF ); }
	then		{ return token( sym.THEN ); }
	else		{ return token( sym.ELSE ); }
	fi			{ return token( sym.FI ); }
	";"			{ return token( sym.SEMICOLON ); }
	","			{ return token( sym.COMMA ); }
	"="			{ return token( sym.ASSIGN ); }

	or			{ return token( sym.OR ); }
	and			{ return token( sym.AND ); }
	not			{ return token( sym.NOT ); }
	"<"			{ return token( sym.LT ); }
	">"			{ return token( sym.GT ); }
	"<="		{ return token( sym.LE ); }
	">="		{ return token( sym.GE ); }
	"=="		{ return token( sym.EQ ); }
	"<>"		{ return token( sym.NE ); }
	"+"			{ return token( sym.PLUS ); }
	"-"			{ return token( sym.MINUS ); }
	"*"			{ return token( sym.TIMES ); }
	"/"			{ return token( sym.DIVIDE ); }
	"("			{ return token( sym.LEFT ); }
	")"			{ return token( sym.RIGHT ); }

	{number}	{ return token( sym.NUMBER ); }
	{string}	{ return token( sym.STRING ); }
	{ident}		{ return token( sym.IDENT ); }

	.			{
				yybegin( LEXERROR ); 
				return token( sym.error ); 
				}
	}
	
<LEXERROR> {
	";"			{
				yybegin( NORMAL ); 
				return token( sym.SEMICOLON ); 
				}
	{newline}	{
				lineNumber++; 
				yybegin( NORMAL ); 
				}
	.			{ }
	}

<<EOF>>			{ return token( sym.EOF ); }

