package node.exprNode.primaryNode;
import runEnv.*;
import node.exprNode.*;
import node.exprNode.valueNode.*;
import node.declNode.*;
import node.declNode.declaratorNode.*;
import java.util.*;

public class InvocationNode extends PrimaryNode {

	private ExprNode function;
	private ExprListNode actualParamList;

	public InvocationNode( ExprNode function, ExprListNode actualParamList ) {
		this.function = function;
		this.actualParamList = actualParamList;
		}
	
	public String toString() {
		return function.toString( ExprNode.PREC_PRIMARY ) + "( " + actualParamList + " )";
		}

	public Object eval( RunEnv runEnv ) {
		FunctionValue functionValue = Cast.functionValue( function.eval( runEnv ) );
		FunctionValueNode functionValueNode = functionValue.functionValueNode();
		RunEnv globals = functionValue.globals();
		Vector actualParams = actualParamList.eval( runEnv );
		IdentListNode formalParams = functionValueNode.formalParams();
		DeclNode locals = functionValueNode.locals();
		Hashtable< String, Object > localDecls = new Hashtable< String, Object >();
		if ( formalParams.size() != actualParams.size() )
			error( "Wrong number of actual parameters" );
		for ( int i = 0; i < formalParams.size(); i++ ) {
			String ident = formalParams.elementAt( i );
			if ( localDecls.get( ident ) != null )
				error( ident + " redeclared" );
			localDecls.put( ident, actualParams.elementAt( i ) );
			}
		RunEnv localEnv = 
				new RunEnv( localDecls, functionValue.globals() );
		locals.eval( localDecls, localEnv );
		try {
			functionValueNode.body().eval( localEnv );
			return new NullValue();
			}
		catch ( ReturnException exception ) {
			return exception.returnValue();
			}
		}
	}
