package node.exprNode;
import env.*;
import type.*;
import runEnv.*;
import node.*;
import java.util.*;

public class ExprListNode extends ListNode< ExprNode > {

	public ExprListNode() {
		super( ", " );
		}
	
	public ExprListNode( ExprNode node ) {
		this();
		addElement( node );
		}
		
	public void checkType( Env env ) {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.checkType( env );
			}
		}
		
	public void castArray( ArrayType arrayType ) {
		Type subType = arrayType.subType();
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			setElementAt( i, expr.castTo( subType ) );
			}
		}

	public void castMultiple( StructType structType ) {
		if ( structType.fieldDecls().size() != size() )
			error( "Wrong number of components to a multiple" );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			Type fieldType = structType.fieldDecls().elementAt( i ).type();
			setElementAt( i, expr.castTo( fieldType ) );
			}
		}

	public void castParams( FunctType functType ) {
		if ( functType.formalDecls().size() != size() )
			error( "Wrong number of actual parameters" );
		for ( int i = 0; i < size(); i++ ) {
			//	Print.error().println( "i = " + i );
			ExprNode expr = elementAt( i );
			int kind = functType.formalDecls().elementAt( i ).kind();
			Type paramType = functType.formalDecls().elementAt( i ).type();
			//	Print.error().println( "paramType = " + paramType );
			switch ( kind ) {
				case Decl.VALUEPARAM:
					setElementAt( i, expr.castTo( paramType ) );
					break;
				case Decl.VARPARAM:
					if ( ! expr.type().equals( paramType ) )
						error( "Invalid type for var parameter" );
					if ( ! ( expr instanceof VarExprNode ) )
						error( "Var parameter not variable" );
					break;
				default:
					error( "Compiler Error - invalid kind" );
				}
			}
		}

	public Multiple eval( RunEnv runEnv ) {
		Multiple multiple = new Multiple( size() );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			RunValue exprValue = expr.eval( runEnv );
			multiple.setValue( i, exprValue );
			}
		return multiple;
		}

	public Multiple evalParams( RunEnv runEnv, FunctType functType ) {
		Multiple multiple = new Multiple( size() );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			int kind = functType.formalDecls().elementAt( i ).kind();
			switch ( kind ) {
				case Decl.VALUEPARAM:
					RunValue exprValue = expr.eval( runEnv );
					multiple.setValue( i, exprValue );
					break;
				case Decl.VARPARAM:
					VarExprNode varExpr = ( VarExprNode ) expr;
					RunData data = varExpr.variable().eval( runEnv );
					multiple.setReference( i, data );
					break;
				default:
					error( "Compiler Error - invalid kind" );
				}
			}
		return multiple;
		}
	}
