package node.exprNode.binaryNode.relationNode;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import node.exprNode.*;

public class EqualNode extends RelationNode {

	Type operandType;

	public EqualNode( ExprNode left, ExprNode right ) {
		super( left, right );
		operator = "==";
		}

	public Type checkType( Env env ) {
		Type leftType = left.checkType( env );
		Type rightType = right.checkType( env );
		if ( leftType.equals( rightType ) )
			operandType = leftType;
		else if ( ( leftType instanceof ArrayType || leftType instanceof StructType )
			&& rightType instanceof NullType )
			operandType = leftType;
		else if ( ( rightType instanceof ArrayType || rightType instanceof StructType )
			&& leftType instanceof NullType )
			operandType = rightType;
		else {
			operandType = Type.balanceType( leftType, rightType );
			if ( operandType == null )
				error( "Incompatible operand types of " + leftType + 
					" and " + rightType + " for ==" );
			left = left.castTo( operandType );
			right = right.castTo( operandType );
			}
		type = BoolType.type;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		if ( operandType instanceof IntType )
			return new RunValue( leftValue.intValue() == rightValue.intValue() );
		else if ( operandType instanceof CharType )
			return new RunValue( leftValue.charValue() == rightValue.charValue() );
		else if ( operandType instanceof BoolType )
			return new RunValue( leftValue.boolValue() == rightValue.boolValue() );
		else if ( operandType instanceof RealType )
			return new RunValue( leftValue.realValue() == rightValue.realValue() );
		else if ( operandType instanceof ArrayType || operandType instanceof StructType )
			return new RunValue( leftValue.multipleValue() == rightValue.multipleValue() );
		error( "Invalid Type for ==" );
		return null;
		}
	}
