package node.exprNode.variableNode;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import node.exprNode.*;

public class SubscriptVariableNode extends VariableNode {

	private ExprNode array;
	private ExprNode subscript;
	
	public SubscriptVariableNode( ExprNode array, ExprNode subscript ) {
		this.array = array;
		this.subscript = subscript;
		}

	public Type checkType( Env env ) {
		Type aType = array.checkType( env );
		if ( ! ( aType instanceof ArrayType ) )
			error( "Attempting to subscript a non-array" );
		ArrayType arrayType = ( ArrayType ) aType;
		Type subscriptType = subscript.checkType( env );
		subscript = subscript.castTo( IntType.type );
		type = arrayType.subType();
		return type;
		}

	public String toString() {
		return array.toString( ExprNode.PREC_PRIMARY ) + "[ " + subscript + " ]";
		}

	public RunData eval( RunEnv runEnv ) {
		RunValue arrayValue = array.eval( runEnv );
		RunValue subscriptValue = subscript.eval( runEnv );
		return arrayValue.multipleValue().getData( subscriptValue.intValue() );
		}
	}
