package node.stmtNode;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import node.exprNode.*;

public class ForStmtNode extends StmtNode {

	private ExprNode initial;
	private ExprNode cond;
	private ExprNode increment;
	private StmtListNode stmtList;

	public ForStmtNode( 
		ExprNode initial, 
		ExprNode cond, 
		ExprNode increment, 
		StmtListNode stmtList ) {
		this.initial = initial;
		this.cond = cond;
		this.increment = increment;
		this.stmtList = stmtList;
		}

	public String toString() {
		return "for " + initial + "; " + cond + "; " 
			+ increment + " do%+%n" + stmtList + "%-%nend";
		}

	public void checkType( Env env ) {
		Type initialType = initial.checkType( env );
		initial = initial.castTo( VoidType.type );
		Type condType = cond.checkType( env );
		cond = cond.castTo( BoolType.type );
		Type incrementType = increment.checkType( env );
		increment = increment.castTo( VoidType.type );
		stmtList.checkType( env );
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		for ( initial.eval( runEnv ); cond.eval( runEnv ).boolValue(); increment.eval( runEnv ) )
			stmtList.eval( runEnv );
		}

	}
