package env;

import node.*;
import node.declNode.*;
import type.*;
import text.*;

public class Env {

	public final static int ENV_NONE		= 0;
	public final static int ENV_LABEL		= 1;
	public final static int ENV_GLOBAL		= 2;
	public final static int ENV_METHOD		= 3;
	public final static int ENV_LOCAL		= 4;
	public final static int ENV_CLASS		= 5;

	private int envKind;
	private Env enclosingEnv;
	private Offset offset = new Offset( 0 );
	private Decl decl;
	private DeclList localDecls = new DeclList();
	private int nestingLevel;

	public Env enclosingEnv() { return enclosingEnv; }
	public Offset offset() { return offset; }
	public Decl decl() { return decl; }
	public DeclList localDecls() { return localDecls; }
	public int nestingLevel() { return nestingLevel; }

	public Env( int envKind, Env enclosingEnv ) {
		this.envKind = envKind;
		this.enclosingEnv = enclosingEnv;
		if ( enclosingEnv == null )
			nestingLevel = 0;
		else
			nestingLevel = enclosingEnv.nestingLevel() + 1;
//		Print.error().println( "Env " + this );
		}
	
	public Env( int envKind ) {
		this( envKind, null );
		}
	
	public static String envKindText( int envKind ) {
		switch ( envKind ) {
			case ENV_NONE: return "ENV_NONE";
			case ENV_LABEL: return "ENV_LABEL";
			case ENV_GLOBAL: return "ENV_GLOBAL";
			case ENV_METHOD: return "ENV_METHOD";
			case ENV_LOCAL: return "ENV_LOCAL";
			case ENV_CLASS: return "ENV_CLASS";
			default: return "ENV_" + envKind;
			}
		}
	
	public String toString() {
		return "Env " + envKindText( envKind ) + " " + decl;
		}
		
	public Decl methodDecl() {
		//	Used for return statements
//		Print.error().println( "methodDecl " + this );
		if ( envKind == ENV_METHOD )
			return decl;
		else if ( enclosingEnv == null )
			return null; 
		else
			return enclosingEnv.methodDecl();
		}

/*
These methods to search for an identifier 
*/
	public Decl searchLocal( String ident ) {
		return localDecls.searchDeclList( ident );
		}
		
	public Decl searchEnv( String ident ) {
		Decl localDecl;
		localDecl = searchLocal( ident );
		if ( localDecl != null )
			return localDecl;
		Decl enclosingDecl;
		if ( enclosingEnv == null )
			enclosingDecl = null;
		else
			enclosingDecl = enclosingEnv.searchEnv( ident );
		return enclosingDecl;
		}

	public void setDecl( Decl decl ) {
		this.decl = decl;
//		Print.error().println( "setDecl " + this );
		}

	public Decl declare( 
		String ident, 
		int section, 
		DeclNode declNode ) {
		return localDecls.declare( ident, section, declNode, this );
		}
	
	public DeclList paramDecls() {
		DeclList paramDecls = new DeclList();
		for ( int i = 0; i < localDecls.size(); i++ ) {
			Decl decl = localDecls.elementAt( i );
			switch ( decl.section() ) {
				case Decl.SECT_VARPARAM:
				case Decl.SECT_VALUEPARAM:
					paramDecls.addElement( decl );
				}
			}
		return paramDecls;
		}

	}
