package node.exprNode;
import node.*;
import node.exprNode.variableNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import java.util.*;

public class ExprListNode extends ListNode< ExprNode > {

	public ExprListNode() {
		super( ", " );
		}
	
	public ExprListNode( ExprNode node ) {
		this();
		addElement( node );
		}
		
	public void genEnv( Env env ) {
		super.genEnv( env );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.genEnv( env );
			}
		}
		
	public void checkType() {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.checkType();
			}
		}
		
	public void castArray( ArrayType arrayType ) {
		Type elementType = arrayType.subType();
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			setElementAt( i, expr.castTo( elementType ) );
			}
		}

	public void castInstance( ClassInstanceType instanceType ) {
		if ( instanceType.fieldDecls().size() != size() )
			error( "Wrong number of fields" );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			Type fieldType = instanceType.fieldDecls().elementAt( i ).type();
			setElementAt( i, expr.castTo( fieldType ) );
			}
		}

	public void castParams( MethodType methodType ) {
		if ( methodType.formalDecls().size() != size() )
			error( "Wrong number of actual parameters" );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			int section = methodType.formalDecls().elementAt( i ).section();
			Type paramType = methodType.formalDecls().elementAt( i ).type();
			switch ( section ) {
				case Decl.SECT_VALUEPARAM:
					setElementAt( i, expr.castTo( paramType ) );
					break;
				case Decl.SECT_VARPARAM:
					if ( ! ( expr instanceof VarExprNode ) )
						error( "Var parameter not variable" );
					VarExprNode varExpr = ( VarExprNode ) expr;
					if ( ! varExpr.variable().addressable() )
						error( "Var parameter not variable" );
					if ( ! expr.type().equals( paramType ) )
						error( "Invalid type for var parameter" );
					break;
				default:
					error( "Compiler Error - invalid section" );
				}
			}
		}

	public void evalVoid( RunEnv runEnv ) {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.eval( runEnv );
			}
		}

	public Multiple evalMultiple( RunEnv runEnv ) {
		Multiple multiple = new Multiple( size() );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			RunValue exprValue = expr.eval( runEnv );
			multiple.setValue( i, exprValue );
			}
		return multiple;
		}

	public Multiple evalParams( RunEnv runEnv, MethodType methodType ) {
		Multiple multiple = new Multiple( size() );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			int section = methodType.formalDecls().elementAt( i ).section();
			switch ( section ) {
				case Decl.SECT_VALUEPARAM:
					RunValue exprValue = expr.eval( runEnv );
					multiple.setValue( i, exprValue );
					break;
				case Decl.SECT_VARPARAM:
					VarExprNode varExpr = ( VarExprNode ) expr;
					RunAddress data = varExpr.variable().evalAddress( runEnv );
					multiple.setValue( i, data );
					break;
				default:
					error( "Compiler Error - invalid section" );
				}
			}
		return multiple;
		}

	}
