package node.exprNode;
import node.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public abstract class ExprNode extends Node {

	public final static int PREC_ASSIGN	=	0;
	public final static int PREC_OR		=	1;
	public final static int PREC_AND		=	2;
	public final static int PREC_REL		=	3;
	public final static int PREC_ADD		=	4;
	public final static int PREC_MUL		=	5;
	public final static int PREC_PREFIX	=	6;
	public final static int PREC_POSTFIX	=	7;
	public final static int PREC_PRIMARY	=	8;

	protected int precedence;
	protected Type type;
	public Type type() { return type; }

	public String toString( int parentPrec ) {
		if ( precedence < parentPrec )
			return "( " + toString() + " )";
		else
			return toString();
		}

	public abstract Type checkType();
	public abstract RunValue eval( RunEnv runEnv );

	public ExprNode castTo( Type postType ) {
		if ( postType == null )
			error( "Compiler Error - null type in castTo" );
		if ( type.equals( postType ) )
			return this;
		else if ( postType.isCastable( type ) )
			return new CastNode( postType, this );
		error( "Can't cast " + type + " to " + postType );
		return this;
		}
		
	}
