package node.exprNode.binaryNode.relationNode;
import node.*;
import node.exprNode.*;
import node.exprNode.binaryNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import text.*;
import java.util.*;

public class EqualNode extends RelationNode {

	public EqualNode( ExprNode left, ExprNode right ) {
		super( left, right );
		operator = "==";
		}

	public Type checkType() {
		Type leftType = left.checkType();
		Type rightType = right.checkType();
		operandType = Type.balanceType( leftType, rightType );
		if ( operandType == null )
			error( "Incompatible operand types of " + leftType + 
				" and " + rightType + " for !=" );
		left = left.castTo( operandType );
		right = right.castTo( operandType );
		type = BoolType.type;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		if ( operandType instanceof CharType )
			return new BoolValue( leftValue.charValue() == rightValue.charValue() );
		else if ( operandType instanceof IntType )
			return new BoolValue( leftValue.intValue() == rightValue.intValue() );
		else if ( operandType instanceof CharType )
			return new BoolValue( leftValue.charValue() == rightValue.charValue() );
		else if ( operandType instanceof BoolType )
			return new BoolValue( leftValue.boolValue() == rightValue.boolValue() );
		else if ( operandType instanceof RealType )
			return new BoolValue( leftValue.realValue() == rightValue.realValue() );
		else if ( operandType instanceof NullType )
			return new BoolValue( true );
		else if ( operandType instanceof ArrayType )
			return new BoolValue( leftValue == rightValue );
		else if ( operandType instanceof ClassInstanceType )
			return new BoolValue( leftValue == rightValue );
		error( "Invalid Type for ==" );
		return null;
		}

	}
