package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.typeNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import text.*;

public class CreationNode extends PrimaryNode {

	private TypeNode typeNode;

	public CreationNode( TypeNode typeNode ) {
		this.typeNode = typeNode;
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		typeNode.genEnv( env );
		}

	public Type checkType() {
		type = typeNode.getType();
		if ( type == null )
			error( "Invalid type" );
		return type;
		}

	public String toString() {
		return "new " + typeNode;
		}

	public RunValue eval( RunEnv runEnv ) {
		if ( type instanceof IntType ) {
			return new IntValue( 0 );
			}
		else if ( type instanceof CharType ) {
			return new CharValue( '\0' );
			}
		else if ( type instanceof RealType ) {
			return new RealValue( 0.0 );
			}
		else if ( type instanceof BoolType ) {
			return new BoolValue( false );
			}
		else if ( type instanceof StringType ) {
			return new StringValue( "" );
			}
		else if ( type instanceof ArrayType ) {
			return new ArrayValue( new Multiple( 0 ) );
			}
		else if ( type instanceof ClassInstanceType ) {
			ClassInstanceType structType = ( ClassInstanceType ) type;
			int size = structType.env().offset().value();
			return new InstanceValue( new Multiple( size ) );
			}
		error( "Invalid Type for new" );
		return null;
		}
	}
