package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.typeNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import java.util.*;

public class MultipleNode extends PrimaryNode {

	private TypeNode typeNode;
	private ExprListNode exprList;

	public MultipleNode( TypeNode typeNode, ExprListNode exprList ) {
		this.typeNode = typeNode;
		this.exprList = exprList;
		}

	public String toString() {
		if ( exprList.size() == 0 )
			return "new " + typeNode + "{}";
		else
			return "new " + typeNode + "{ " + exprList + " }";
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		typeNode.genEnv( env );
		exprList.genEnv( env );
		}

	public Type checkType() {
		type = typeNode.getType();
		exprList.checkType();
		if ( type instanceof ArrayType )
			exprList.castArray( ( ArrayType ) type );
		else if ( type instanceof ClassInstanceType )
			exprList.castInstance( ( ClassInstanceType ) type );
		else
			error( "Multiple must be an array or class instance" );
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		if ( type instanceof ArrayType ) {
			ArrayType arrayType = ( ArrayType ) type;
			Type subType = arrayType.subType();
			Multiple multiple = exprList.evalMultiple( runEnv );
			if ( subType instanceof CharType )
				return new StringValue( multiple );
			else
				return new ArrayValue( multiple );
			}
		else if ( type instanceof ClassInstanceType ) {
			Multiple multiple = exprList.evalMultiple( runEnv );
			return new InstanceValue( multiple );
			}
		return null;
		}
	}
