package node.exprNode.variableNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class IndexVariableNode extends VariableNode {

	private ExprNode array;
	private ExprNode index;
	
	public IndexVariableNode( ExprNode array, ExprNode index ) {
		this.array = array;
		this.index = index;
		}

	public String toString() {
		return array.toString( ExprNode.PREC_PRIMARY ) + "[ " + index + " ]";
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		array.genEnv( env );
		index.genEnv( env );
		}

	public Type checkType() {
		Type aType = array.checkType();
		if ( ! ( aType instanceof ArrayType ) )
			error( "Attempting to index a non-array" );
		ArrayType arrayType = ( ArrayType ) aType;
		Type indexType = index.checkType();
		index = index.castTo( IntType.type );
		type = arrayType.subType();
		return type;
		}

	public RunAddress evalAddress( RunEnv runEnv ) {
		RunValue arrayValue = array.eval( runEnv );
		RunValue indexValue = index.eval( runEnv );
		if ( arrayValue instanceof NullValue )
			throw new Error( "Attempting to index null array" );
		return arrayValue.arrayValue().getAddress( indexValue.intValue() );
		}

	}
