package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class ForStmtNode extends StmtNode {

	private DeclStmtNode initial;
	private ExprNode cond;
	private ExprListNode increment;
	private StmtNode stmt;
	private Env localEnv;
	
	public ForStmtNode( 
		DeclStmtNode initial, 
		ExprNode cond, 
		ExprListNode increment, 
		StmtNode stmt ) {
		this.initial = initial;
		this.cond = cond;
		this.increment = increment;
		this.stmt = stmt;
		}

	public String toString() {
		return "for ( " + initial + " " + cond + "; " 
			+ increment + " )%+%n" + stmt + "%-";
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		localEnv = new Env( Env.ENV_LOCAL, env );
		initial.genEnv( localEnv );
		cond.genEnv( localEnv );
		increment.genEnv( localEnv );
		stmt.genEnv( localEnv );
		}

	public void setType() {
		initial.setType();
		stmt.setType();
		}

	public void checkType() {
		initial.checkType();
		Type condType = cond.checkType();
		cond = cond.castTo( BoolType.type );
		increment.checkType();
		stmt.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws UserException {
		Multiple locals = new Multiple( localEnv.offset().value() );
		RunEnv localRunEnv = new RunEnv( locals, runEnv );
		try {
			for ( 
				initial.eval( localRunEnv ); 
				cond.eval( localRunEnv ).boolValue(); 
				increment.evalVoid( localRunEnv ) )
				try {
					stmt.eval( localRunEnv );
					}
				catch ( ContinueException exception ) {
					}
			}
		catch ( BreakException exception ) {
			}
		}

	}
