package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class IfElseStmtNode extends StmtNode {

	private ExprNode cond;
	private StmtNode thenPart;
	private StmtNode elsePart;

	public IfElseStmtNode( ExprNode cond, StmtNode thenPart, StmtNode elsePart ) {
		this.cond = cond;
		this.thenPart = thenPart;
		this.elsePart = elsePart;
		}

	public String toString() {
		return "if ( " + cond + " )%+%n" + thenPart + "%-%nelse%+%n" + elsePart + "%-";
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		cond.genEnv( env );
		thenPart.genEnv( env );
		elsePart.genEnv( env );
		}

	public void setType() {
		thenPart.setType();
		elsePart.setType();
		}

	public void checkType() {
		Type condType = cond.checkType();
		cond = cond.castTo( BoolType.type );
		thenPart.checkType();
		elsePart.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws UserException {
		if ( cond.eval( runEnv ).boolValue() )
			thenPart.eval( runEnv );
		else
			elsePart.eval( runEnv );
		}

	}
