package runEnv;

import env.*;
import type.*;

public class RunEnv {

	private Multiple locals;
	private RunEnv enclosingEnv;
	private int nestingLevel;
	
	public Multiple locals() { return locals; }
	public RunEnv enclosingEnv() { return enclosingEnv; }
	public int nestingLevel() { return nestingLevel; }
	
	public RunEnv( 
		Multiple locals,
		RunEnv enclosingEnv ) {
		this.locals = locals;
		this.enclosingEnv = enclosingEnv;
		if ( enclosingEnv == null )
			nestingLevel = 0;
		else
			nestingLevel = enclosingEnv.nestingLevel() + 1;
		}

	public RunEnv globals() {
		if ( enclosingEnv == null )
			return this;
		else
			return enclosingEnv.globals();
		}

	public RunEnv enclosingEnv( int nestingLevel ) {
		if ( this.nestingLevel == nestingLevel )
			return this;
		else if ( enclosingEnv == null )
			return null;
		else
			return enclosingEnv.enclosingEnv( nestingLevel );
		}
	
	public Multiple locals( int nestingLevel ) {
		return enclosingEnv( nestingLevel ).locals();
		}
	
	public RunAddress getAddress( Decl decl ) {
		switch ( decl.section() ) {
			case Decl.SECT_VARPARAM:
				return locals( decl.nestingLevel() ).getValue( 
					decl.offset() ).addressValue();
			case Decl.SECT_VALUEPARAM:
			case Decl.SECT_VARIABLE:
				return locals( decl.nestingLevel() ).getAddress( 
					decl.offset() );
			default:
				throw new Error( "Invalid section of " 
					+ Decl.sectionText( decl.section() ) );
			}
		}
	}
