import java.util.*;

import java.io.*;
import java_cup.runtime.*;
import grammar.*;
import env.*;
import type.*;
import node.*;
import template.*;
import text.*;
import runEnv.*;

public class Main {

	public static void main( String[] argv ) {
		try {
			boolean reprint = false;
			boolean check = false;
			boolean interp = false;
			boolean compile = false;
			String dirName = null;
			for ( int i = 0; i < argv.length; i++ ) {
				if ( argv[ i ].equals( "-debug" ) ) {
					Print.DEBUG = true;
					}
				else if ( argv[ i ].equals( "-dir" ) ) {
					i++;
					if ( i >= argv.length )
						throw new Error( "Missing directory name" );
					dirName = argv[ i ];
					}
				else if ( argv[ i ].equals( "-reprint" ) ) {
					reprint = true;
					}
				else if ( argv[ i ].equals( "-check" ) ) {
					check = true;
					}
				else if ( argv[ i ].equals( "-interp" ) ) {
					interp = true;
					}
				else if ( argv[ i ].equals( "-compile" ) ) {
					compile = true;
					}
				else {
					throw new Error( 
						"Usage Error: java Main -dir directory [-reprint] [-check] [ -interp ] [ -compile ] on argument " + i );
					}
				}

			if ( dirName == null )
				throw new Error( "Directory not specified" );
			if ( interp || compile )
				check = true;

			Print.setError( new File( dirName, "program.err" ) );

			File inputFile = new File( dirName, "program.in" );

			parser p = new parser( inputFile );
			ProgramNode program = ( ProgramNode ) ( p.parse().value );
			if ( reprint ) {
				Print.error().println( "Reprinting ... " );
				Print.setReprint( new File( dirName, "program.print" ) );
				Print.reprint().println( program );
				}
			if ( check ) {
				Print.error().println( "Generate Environments ... " );
				program.genEnv();
				Print.error().println( "Generate Declarations ... " );
				program.genDecl();
				Print.error().println( "Check Types ... " );
				program.checkType();
				Print.error().println( "Calculate Weights ... " );
				program.calcWeight();
				}
			if ( interp ) {
				Print.error().println( "Evaluate ... " );
				Print.setInterp( new File( dirName, "interp.out" ) );
				try {
					program.eval();
					}
				catch ( ExitException exception ) {
					Print.error().println( "Exit with status " + exception.returnValue() );
					}
				catch ( Error exception ) {
					Print.error().println( "User Error " + exception.getMessage() );
					}
				}
			if ( compile ) {
				Print.error().println( "Generate Code ... " );
				Print.setCompile( new File( dirName, "usercode.user.s" ) );
				program.genCode();
				}
			}
		catch ( Throwable exception ) {
			exception.printStackTrace();
			Print.error().println( "Exception in Main " + exception );
			Print.error().printStackTrace( exception );
			System.exit( -1 );
			}
		}
	}
