package node;
import env.*;
import runEnv.*;
import text.*;

public class DeclStmtListNode extends ListNode< DeclStmtNode > {

	public DeclStmtListNode() {
		super( "%n" );
		}
	
	public DeclStmtListNode( DeclStmtNode node ) {
		this();
		addElement( node );
		}
		
	private int weight;
	public int weight() { return weight; }

	public void genEnv( int section, Env env ) {
		for ( int i = 0; i < size(); i++ ) {
			try {
				DeclStmtNode declStmt = elementAt( i );
				declStmt.genEnv( section, env );
				}
			catch ( Error exception ) {
				Print.error().println( exception.getMessage() );
				setElementAt( i, new ErrorDeclStmtNode() );
				}
			}
		}

	public void genDecl() {
		for ( int i = 0; i < size(); i++ ) {
			try {
				DeclStmtNode declStmt = elementAt( i );
				declStmt.genDecl();
				}
			catch ( Error exception ) {
				Print.error().println( exception.getMessage() );
				setElementAt( i, new ErrorDeclStmtNode() );
				}
			}
		}

	public void checkType() {
		for ( int i = 0; i < size(); i++ ) {
			try {
				DeclStmtNode declStmt = elementAt( i );
				declStmt.checkType();
				}
			catch ( Error exception ) {
				Print.error().println( exception.getMessage() );
				setElementAt( i, new ErrorDeclStmtNode() );
				}
			}
		}

	public void eval( RunEnv runEnv ) throws ReturnException {
		for ( int i = 0; i < size(); i++ ) {
			DeclStmtNode declStmt = elementAt( i );
			declStmt.eval( runEnv );
			}
		}

	public int calcWeight() {
		weight = 0;
		for ( int i = 0; i < size(); i++ ) {
			DeclStmtNode declStmt = elementAt( i );
			int stmtWeight = declStmt.calcWeight();
			weight = max( weight, stmtWeight );
			}
		return weight;
		}

	public void genDeclCode() {
		for ( int i = 0; i < size(); i++ ) {
			DeclStmtNode declStmt = elementAt( i );
			declStmt.genDeclCode();
			}
		}

	public void genCode() {
		for ( int i = 0; i < size(); i++ ) {
			DeclStmtNode declStmt = elementAt( i );
			declStmt.genCode();
			}
		}

	}

