package node.declNode;
import node.*;
import node.typeNode.*;
import env.*;
import type.*;
import type.basicType.*;
import template.*;
import runEnv.*;

public class MethodDeclNode extends DeclStmtNode implements DeclNode {

	private DeclStmtListNode formalParams;
	private DeclStmtListNode body;
	public DeclStmtListNode formalParams() { return formalParams; }
	public DeclStmtListNode body() { return body; }

	private int section;
	private Env env;
	private Env localEnv;
	private Type returnType;
	private Decl decl;
	private int bodyWeight;

	public int section() { return section; }
	public Env env() { return env; }
	public Env localEnv() { return localEnv; }
	public Type returnType() { return returnType; }
	public Decl decl() { return decl; }

	private TypeNode returnTypeNode;
	private String ident;

	public MethodDeclNode( 
		TypeNode returnTypeNode,
		String ident,
		DeclStmtListNode formalParams,
		DeclStmtListNode body ) {
		this.formalParams = formalParams;
		this.body = body;
		this.returnTypeNode = returnTypeNode;
		this.ident = ident;
		}
	
	public String toString() {
		return returnTypeNode + " " + ident 
			+ " (%+%n" + formalParams() + " %-) " + "%+" 
			+ "%nbegin%+%n" + body() + "%-%nend%-";
		}
		
	public void genEnv( int section, Env env ) {
		this.section = section;
		this.env = env;
		localEnv = new Env( Env.ENV_METHOD, env );
		formalParams.genEnv( Decl.SECT_LOCAL, localEnv );
		body.genEnv( Decl.SECT_LOCAL, localEnv );
		returnTypeNode.genEnv( env );
		}

	public void genDecl() {
		returnType = returnTypeNode.getType();
		formalParams().genDecl();
		TypeList typeList = localEnv().paramDecls().typeList();
		decl = env().searchLocal( ident, new InvocationTemplate( typeList ), Decl.PRIVATE );
		if ( decl != null )
			error( "Method " + ident + " redeclared in same environment" );
		decl = env().searchExtended( ident, new InvocationTemplate( typeList ), Decl.PRIVATE );
		if ( decl != null && ! decl.type().equals( 
			new MethodType( returnType, localEnv().paramDecls() ) ) )
				error( "Invalid type for overridden method" );
		switch ( section() ) {
			case Decl.SECT_GLOBAL:
			case Decl.SECT_STATIC:
				decl = env().declare( ident, section(), Decl.PUBLIC,
					new MethodType( returnType, localEnv().paramDecls() ), this );
				break;
			case Decl.SECT_INSTANCE:
				decl = env().declare( ident, Decl.SECT_INSTANCE, Decl.PUBLIC,
					new MethodType( returnType, localEnv().paramDecls() ), this );
				break;
			case Decl.SECT_CLASS:
			case Decl.SECT_VARPARAM:
			case Decl.SECT_LOCAL:
			case Decl.SECT_VALUEPARAM:
			default:
				error( "Invalid section of " + Decl.sectionText( section() ) );
				break;
			}
		localEnv().setDecl( decl );
		body().genDecl();
		}

	public void checkType() {
		formalParams.checkType();
		body.checkType();
		}

	public void eval( RunEnv runEnv ) {
		}

	public int calcWeight() {
		bodyWeight = body.calcWeight();
		localEnv.setLocalOffset();
		weight = 1;
		return weight;
		}

	public void genDeclCode() {
		enterBlock( decl.declName(), "proc.sav" + max( bodyWeight - Node.MAXREG, 0 ), "proc" );
		localEnv.genLocalDecls();
		enter( "code" );
		align();
		publicLabelDefn( "enter" );
		loadA( "$sp", displacement( "-params", "$sp" ) );
		store( "$ra", displacement( "savRA", "$sp" ) );
		store( "$fp", displacement( "savFP", "$sp" ) );
		store( "$ip", displacement( "savIP", "$sp" ) );
		move( "$nip", "$ip" );
		move( "$sp", "$fp" );
		body.genCode();
		if ( returnType instanceof RealType )
			clearF( "$ft0" );
		else
			clear( "$t0" );
		labelDefn( "return" );
		move( "$ip", "$nip" );
		load( "$ip", displacement( "savIP", "$sp" ) );
		load( "$fp", displacement( "savFP", "$sp" ) );
		load( "$ra", displacement( "savRA", "$sp" ) );
		loadA( "$sp", displacement( "+params", "$sp" ) );
		ret();
		exit();
		exit();
		}

	public void genCode() {
		}

	}
