package node.declNode.declaratorNode;
import node.*;
import text.*;
import env.*;
import type.*;
import runEnv.*;

public class DeclaratorListNode extends ListNode< DeclaratorNode > {

	public DeclaratorListNode() {
		super( ", " );
		}
	
	public DeclaratorListNode( DeclaratorNode node ) {
		this();
		addElement( node );
		}
		
	private int weight;
	
	public void genEnv( int section, Env env ) {
		for ( int i = 0; i < size(); i++ ) {
			DeclaratorNode decltr = elementAt( i );
			decltr.genEnv( section, env );
			}
		}

	public void genDecl( Type type ) {
		for ( int i = 0; i < size(); i++ ) {
			try {
				DeclaratorNode decltr = elementAt( i );
				decltr.genDecl( type );
				}
			catch ( Error exception ) {
				Print.error().println( exception.getMessage() );
				setElementAt( i, new ErrorDeclaratorNode() );
				}
			}
		}

	public void checkType() {
		for ( int i = 0; i < size(); i++ ) {
			try {
				DeclaratorNode decltr = elementAt( i );
				decltr.checkType();
				}
			catch ( Error exception ) {
				Print.error().println( exception.getMessage() );
				setElementAt( i, new ErrorDeclaratorNode() );
				}
			}
		}

	public void eval( RunEnv runEnv ) throws ReturnException {
		for ( int i = 0; i < size(); i++ ) {
			try {
				DeclaratorNode decltr = elementAt( i );
				decltr.eval( runEnv );
				}
			catch ( Error exception ) {
				Print.error().println( exception.getMessage() );
				setElementAt( i, new ErrorDeclaratorNode() );
				}
			}
		}

	public int calcWeight() {
		weight = 0;
		for ( int i = 0; i < size(); i++ ) {
			DeclaratorNode decltr = elementAt( i );
			int declrWeight = decltr.calcWeight();
			weight = max( weight, declrWeight );
			}
		return weight;
		}

	public void genDeclCode() {
		for ( int i = 0; i < size(); i++ ) {
			DeclaratorNode decltr = elementAt( i );
			decltr.genDeclCode();
			}
		}

	public void genCode() {
		for ( int i = 0; i < size(); i++ ) {
			DeclaratorNode decltr = elementAt( i );
			decltr.genCode();
			}
		}
	}
