package node.exprNode;
import node.*;
import node.exprNode.variableNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class ExprListNode extends ListNode< ExprNode > {

	public ExprListNode() {
		super( ", " );
		}
	
	public ExprListNode( ExprNode node ) {
		this();
		addElement( node );
		}
		
	private int weight;
	public int weight() { return weight; }

	public void genEnv( Env env ) {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.genEnv( env );
			}
		}
		
	public void checkType() {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			expr.checkType();
			}
		}
	
	public TypeList typeList() {
		TypeList typeList = new TypeList();
		for ( int i = 0; i < size(); i++ ) {
			typeList.addElement( elementAt( i ).type() );
			}
		return typeList;
		}

	public void castArray( Type subType ) {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			setElementAt( i, expr.castTo( subType ) );
			}
		}

	public void castParams( MethodType methodType ) {
		if ( methodType.formalDecls().size() != size() )
			error( "Wrong number of actual parameters" );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			int section = methodType.formalDecls().elementAt( i ).section();
			Type paramType = methodType.formalDecls().elementAt( i ).type();
			switch ( section ) {
				case Decl.SECT_VALUEPARAM:
					setElementAt( i, expr.castTo( paramType ) );
					break;
				case Decl.SECT_VARPARAM:
					if ( ! ( expr instanceof VarExprNode ) )
						error( "Var parameter not variable" );
					if ( ! expr.type().equals( paramType ) )
						error( "Invalid type for var parameter" );
					break;
				default:
					error( "Compiler Error - invalid section" );
				}
			}
		}

	public Multiple evalArray( RunEnv runEnv ) {
		Multiple multiple = new Multiple( size() );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			RunValue exprValue = expr.eval( runEnv );
			multiple.setValue( i, exprValue );
			}
		return multiple;
		}

	public Multiple evalParams( RunEnv runEnv, MethodType methodType ) {
		Multiple multiple = new Multiple( size() );
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			int section = methodType.formalDecls().elementAt( i ).section();
			switch ( section ) {
				case Decl.SECT_VALUEPARAM:
					RunValue exprValue = expr.eval( runEnv );
					multiple.setValue( i, exprValue );
					break;
				case Decl.SECT_VARPARAM:
					VarExprNode varExpr = ( VarExprNode ) expr;
					RunData data = varExpr.variable().eval( runEnv );
					multiple.setValue( i, data );
					break;
				default:
					error( "Compiler Error - invalid section" );
				}
			}
		return multiple;
		}
		
	public int calcWeight() {
		weight = 0;
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			int exprWeight = expr.calcWeight();
			weight = max( weight, exprWeight );
			}
		return weight;
		}

	public void genCode( MethodType methodType, int freeTemp ) {
		for ( int i = 0; i < size(); i++ ) {
			ExprNode expr = elementAt( i );
			int section = methodType.formalDecls().elementAt( i ).section();
			Type type = methodType.formalDecls().elementAt( i ).type();
			switch ( section ) {
				case Decl.SECT_VALUEPARAM:
					expr.regCode( 0, freeTemp );
					if ( type instanceof RealType )
						storeF( ftempReg( 0 ), displacement( "invoc.act" + i, "$sp" ) );
					else
						store( tempReg( 0 ), displacement( "invoc.act" + i, "$sp" ) );
					break;
				case Decl.SECT_VARPARAM:
					VarExprNode varExpr = ( VarExprNode ) expr;
					loadA( tempReg( 0 ), varExpr.variable().addrCode( 0, freeTemp ) );
					store( tempReg( 0 ), displacement( "invoc.act" + i, "$sp" ) );
					break;
				default:
					error( "Compiler Error - invalid section" );
				}
			}
		}
	}
