package node.exprNode.methodNameNode;
import node.*;
import node.declNode.*;
import node.exprNode.*;
import env.*;
import type.*;
import template.*;
import runEnv.*;
import runEnv.basicValue.*;

public class MemberMethodNameNode extends MethodNameNode {

	private ExprNode object;
	private Type objectType;
	private String ident;

	public MemberMethodNameNode( ExprNode object, String ident ) {
		this.object = object;
		this.ident = ident;
		}

	public String toString() {
		return object + "." + ident;
		}

	public void genEnv( Env env ) {
		object.genEnv( env );
		}

	public MethodType checkType( TypeList typeList ) {
		objectType = object.checkType();
		if ( objectType instanceof InstanceType ) {
			InstanceType instanceType = ( InstanceType ) objectType;
			decl = instanceType.env().searchExtended( ident, 
				new InvocationTemplate( typeList ), Decl.PRIVATE );
			if ( decl == null )
				error( "Undeclared method " + ident );
			switch ( decl.section() ) {
				case Decl.SECT_INSTANCE:
					type = ( MethodType ) decl.type();
					break;
				default:
					error( "CodeName " + ident + " must be instance method" );
					type = null;
				}
			}
		else if ( objectType instanceof ClassType ) {
			ClassType classType = ( ClassType ) objectType;
			decl = classType.env().searchEnv( ident, 
				new InvocationTemplate( typeList ), Decl.PRIVATE );
			if ( decl == null )
				error( "Undeclared static method " + ident );
			switch ( decl.section() ) {
				case Decl.SECT_STATIC:
					type = ( MethodType ) decl.type();
					break;
				default:
					error( "CodeName " + ident + " must be static method" );
					type = null;
				}
			}
		else
			error( "Selection must be of a class name or instance" );
		return type;
		}

	public MethodValue eval( RunEnv runEnv ) {
		InstanceValue instanceValue;
		switch ( decl.section() ) {
			case Decl.SECT_INSTANCE:
				//	System.err.println( "Setting up methodValue for " + decl.absoluteName() );
				instanceValue = object.eval( runEnv ).instanceValue();
				return new MethodValue( instanceValue.methodTable().elementAt( decl.offset() ), instanceValue );
			case Decl.SECT_STATIC:
				return new MethodValue( decl, null );
			default:
				error( "Invalid section" );
				return null;
			}
		}

	public int calcWeight() {
		weight = object.calcWeight();
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		switch ( decl.section() ) {
			case Decl.SECT_INSTANCE:
				object.regCode( freeReg, freeTemp );
				move( tempReg( freeReg ), "$nip" );
				load( "$at", displacement( "INST.methods", "$nip" ) );
				load( "$pv", displacement( decl.absoluteName(), "$at" ) );
				break;
			case Decl.SECT_STATIC:
				move( "$zero", "$nip" );
				loadIQ( "$pv", decl.absoluteName() + ".enter" );
				break;
			default:
				error( "Invalid section" );
				break;
			}
		}
	}
