package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.declNode.*;
import env.*;
import type.*;
import type.basicType.*;
import template.*;
import runEnv.*;

public class ConstructorNode extends PrimaryNode {

	private String ident;
	private Env env;
	private Decl decl;
	private Env classEnv;
	
	public ConstructorNode( String ident ) {
		this.ident = ident;
		}

	public String toString() {
		return "new " + ident;
		}

	public void genEnv( Env env ) {
		this.env = env;
		}

	public Type checkType() {
		decl = env.searchEnv( ident, ClassTemplate.template, Decl.PRIVATE );
		if ( decl != null ) {
			
			ClassType classType = (ClassType ) decl.type();
			classEnv = classType.env();
			type = decl.type().instanceType();
			return type;
			}
		error( "Undeclared class " + ident );
		return null;
		}

	public RunValue eval( RunEnv runEnv ) {
		InstanceValue instanceValue = new InstanceValue( 
			new Multiple( classEnv.instanceFieldOffset().value() ),
			classEnv.methodTable() );
		ClassDeclNode classDeclNode = ( ClassDeclNode ) decl.declNode();
		classDeclNode.initInstance( 
			new RunEnv( runEnv.currentStatic(), instanceValue, null ) );
		return instanceValue;
		}
		
	public int calcWeight() {
		weight = MAXREG;
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		loadIQ( "$a0", classEnv.absoluteName() + ".object.field.max" );
		loadIQ( "$a1", classEnv.absoluteName() + ".methodTable.enter" );
		bsr( "INST.create.enter" );
		move( "$v0", "$nip" );
		bsr( classEnv.absoluteName() + ".instance.init.enter" );
		move( "$nip", tempReg( freeReg ) );
		}
	}
