package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.declNode.*;
import node.exprNode.methodNameNode.*;
import env.*;
import type.*;
import template.*;
import type.basicType.*;
import runEnv.*;
import text.*;

public class InvocationNode extends PrimaryNode {

	private MethodNameNode methodName;
	private ExprListNode actualParamList;
	private MethodType methodType;

	public InvocationNode( MethodNameNode methodName, ExprListNode actualParamList ) {
		this.methodName = methodName;
		this.actualParamList = actualParamList;
		}
	
	public String toString() {
		return methodName + "( " + actualParamList + " )";
		}

	public void genEnv( Env env ) {
		methodName.genEnv( env );
		actualParamList.genEnv( env );
		}

	public Type checkType() {
		actualParamList.checkType();
		methodType = methodName.checkType( actualParamList.typeList() );
		actualParamList.castParams( methodType );
		type = methodType.returnType();
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		MethodValue methodValue = methodName.eval( runEnv );
		InstanceValue instanceValue = methodValue.instanceValue();
		Decl methodDecl = methodValue.methodDecl();
		MethodDeclNode methodDeclNode = ( MethodDeclNode ) methodDecl.declNode();
		Env localEnv = methodDeclNode.localEnv();
		int localOffset = localEnv.localOffset().value();
		Multiple locals = new Multiple( localOffset );
		Multiple evalParams = actualParamList.evalParams( runEnv, methodType );
		for ( int i = 0; i < actualParamList.size(); i++ )
			locals.setValue( localOffset - actualParamList.size() + i, evalParams.getValue( i ) );
		try {
			InvocationValue invocationValue = new InvocationValue( locals );
			RunEnv invocationRunEnv = 
				new RunEnv( runEnv.currentStatic(), instanceValue, invocationValue );
			methodDeclNode.body().eval( invocationRunEnv );
			return null;
			}
		catch ( ReturnException exception ) {
			return exception.returnValue();
			}
		}

	public int calcWeight() {
		int methodWeight = methodName.calcWeight();
		int paramWeight = actualParamList.calcWeight();
		weight = max( methodWeight, paramWeight + 1, MAXREG );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		loadA( "$sp", displacement( "-invoc.act" + actualParamList.size(), "$sp" ) );
		actualParamList.genCode( methodType, freeTemp );
		methodName.regCode( freeReg, freeTemp );
		jsr();
		loadA( "$sp", displacement( "+invoc.act" + actualParamList.size(), "$sp" ) );
		}
	}
