package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.typeNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class SubArrayNode extends PrimaryNode {

	private ExprNode arrayExpr;
	private ExprNode lowExpr;
	private ExprNode highExpr;

	public SubArrayNode( 
		ExprNode arrayExpr,
		ExprNode lowExpr,
		ExprNode highExpr ) {
		this.arrayExpr = arrayExpr;
		this.lowExpr = lowExpr;
		this.highExpr = highExpr;
		}

	public String toString() {
		return arrayExpr.toString( ExprNode.PREC_PRIMARY ) 
			+ "[ " + lowExpr + " .. " + highExpr + " ]";
		}

	public void genEnv( Env env ) {
		arrayExpr.genEnv( env );
		lowExpr.genEnv( env );
		highExpr.genEnv( env );
		}

	public Type checkType() {
		Type arrayExprType = arrayExpr.checkType();
		Type lowExprType = lowExpr.checkType();
		Type highExprType = highExpr.checkType();
		lowExpr = lowExpr.castTo( IntType.type );
		highExpr = highExpr.castTo( IntType.type );
		type = arrayExprType;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue arrayValue = arrayExpr.eval( runEnv );
		RunValue lowValue = lowExpr.eval( runEnv );
		RunValue highValue = highExpr.eval( runEnv );
		if ( arrayValue instanceof NullValue )
			throw new Error( "Attempting to form subarray of null array" );
		return arrayValue.arrayValue().subArray( 
			lowValue.intValue(), highValue.intValue() );
		}

	public int calcWeight() {
		int arrayWeight = arrayExpr.calcWeight();
		int lowExprWeight = lowExpr.calcWeight();
		int highExprWeight = highExpr.calcWeight();
		weight = max( arrayWeight + 1, lowExprWeight + 2, highExprWeight + 3, MAXREG );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		int arrayWeight = arrayExpr.weight();
		int lowExprWeight = lowExpr.weight();
		int highExprWeight = highExpr.weight();
		if ( arrayWeight + 1 < MAXREG  - freeReg
			&& lowExprWeight + 2 < MAXREG - freeReg 
			&& highExprWeight + 3 < MAXREG - freeReg ) {
			move( 16, "$a0" );
			bsr( "Memory.allocate.enter" );
			move( "$v0", tempReg( freeReg ) );
			arrayExpr.regCode( freeReg + 1, freeTemp );
			lowExpr.regCode( freeReg + 2, freeTemp );
			highExpr.regCode( freeReg + 3, freeTemp );
			}
		else {
			arrayExpr.memCode( freeTemp );
			lowExpr.memCode( freeTemp + 1 );
			highExpr.memCode( freeTemp + 2 );
			move( 16, "$a0" );
			bsr( "Memory.allocate.enter" );
			move( "$v0", tempReg( freeReg ) );
			load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
			load( tempReg( freeReg + 2 ), tempMem( freeTemp + 1 ) );
			load( tempReg( freeReg + 3 ), tempMem( freeTemp + 2 ) );
			}
		load( tempReg( freeReg + 4 ), displacement( 0, tempReg( freeReg + 1 ) ) );
		instrn( "addq", tempReg( freeReg + 3 ), "1" );
		instrn( "blt", tempReg( freeReg + 2 ), "IO.error.enter" );
		instrn( "cmple", tempReg( freeReg + 2 ), 
			tempReg( freeReg + 3 ), tempReg( freeReg + 5 ) );
		instrn( "beq", tempReg( freeReg + 5 ), "IO.error.enter" );
		instrn( "cmple", tempReg( freeReg + 3 ), 
			tempReg( freeReg + 4 ), tempReg( freeReg + 5 ) );
		instrn( "beq", tempReg( freeReg + 5 ), "IO.error.enter" );
		instrn( "subq", tempReg( freeReg + 3 ), 
			tempReg( freeReg + 2 ), tempReg( freeReg + 5 ) );
		store( tempReg( freeReg + 5 ), displacement( 0, tempReg( freeReg ) ) );
		load( tempReg( freeReg + 5 ), displacement( 8, tempReg( freeReg + 1 ) ) );
		instrn( "s8addq", tempReg( freeReg + 2 ), tempReg( freeReg + 5 ), tempReg( freeReg + 5 ) );
		store( tempReg( freeReg + 5 ), displacement( 8, tempReg( freeReg ) ) );
		}
	}
