package node.exprNode.variableNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import template.*;
import runEnv.*;
import text.*;

public class MemberVariableNode extends VariableNode {

	private ExprNode object;
	private String ident;
	private Decl decl;	

	public MemberVariableNode( ExprNode object, String ident ) {
		this.object = object;
		this.ident = ident;
		}

	public String toString() {
		return object.toString( ExprNode.PREC_PRIMARY ) + "." + ident;
		}

	public void genEnv( Env env ) {
		object.genEnv( env );
		}

	public Type checkType() {
		Type oType = object.checkType();
		if ( oType instanceof InstanceType ) {
			InstanceType instanceType = ( InstanceType ) oType;
			decl = instanceType.env().searchEnv( ident, FieldTemplate.template, Decl.PROTECTED );
			if ( decl == null )
				error( "Undeclared instance field " + ident );
			switch ( decl.section() ) {
				case Decl.SECT_INSTANCE:
					type = decl.type();
					break;
				default:
					error( "Identifier " + ident + " must be instance field" );
				}
			}
		else if ( oType instanceof ClassType ) {
			ClassType classType = ( ClassType ) oType;
			decl = classType.env().searchEnv( ident, FieldTemplate.template, Decl.PROTECTED );
			if ( decl == null )
				error( "Undeclared static field " + ident );
			if ( decl.section() != Decl.SECT_STATIC )
				error( "Field " + ident + " must be static field" );
			type = decl.type();
			}
		else
			error( "Selection must be of a class name or instance" );
		return type;
		}

	public RunData eval( RunEnv runEnv ) {
		InstanceValue instanceValue;
		switch ( decl.section() ) {
			case Decl.SECT_INSTANCE:
				instanceValue = object.eval( runEnv ).instanceValue();
				return instanceValue.fields().getData( decl.offset() );
			case Decl.SECT_STATIC:
				return runEnv.currentStatic().fields().getData( decl.offset() );
			default:
				error( "Invalid section" );
				return null;
			}
		}

	public int calcWeight() {
		weight = object.calcWeight();
		return weight;
		}

	public String addrCode( int freeReg, int freeTemp ) {
		switch ( decl.section() ) {
			case Decl.SECT_INSTANCE:
				object.regCode( freeReg, freeTemp );
				load( tempReg( freeReg ), 
					displacement( "INST.fields", tempReg( freeReg ) ) );
				return displacement( decl.absoluteName(), tempReg( freeReg ) );
			case Decl.SECT_STATIC:
				return decl.address( tempReg( freeReg ) );
			default:
				error( "Invalid section" );
				return null;
			}
		}
	}
