package node.exprNode.variableNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class SubscriptVariableNode extends VariableNode {

	private ExprNode array;
	private ExprNode subscript;
	
	public SubscriptVariableNode( ExprNode array, ExprNode subscript ) {
		this.array = array;
		this.subscript = subscript;
		}

	public void genEnv( Env env ) {
		array.genEnv( env );
		subscript.genEnv( env );
		}

	public Type checkType() {
		Type aType = array.checkType();
		if ( ! ( aType instanceof ArrayType ) )
			error( "Attempting to subscript a non-array" );
		ArrayType arrayType = ( ArrayType ) aType;
		Type subscriptType = subscript.checkType();
		subscript = subscript.castTo( IntType.type );
		type = arrayType.subType();
		return type;
		}

	public String toString() {
		return array.toString( ExprNode.PREC_PRIMARY ) + "[ " + subscript + " ]";
		}

	public RunData eval( RunEnv runEnv ) {
		RunValue arrayValue = array.eval( runEnv );
		RunValue subscriptValue = subscript.eval( runEnv );
		return arrayValue.arrayValue().array().getData( subscriptValue.intValue() );
		}

	public int calcWeight() {
		int arrayWeight = array.calcWeight();
		int subscriptWeight = subscript.calcWeight();
		weight = min( 
			max( arrayWeight + 1, subscriptWeight, 3 ),
			max( subscriptWeight + 1, arrayWeight, 3 ) );
		return weight;
		}

	public String addrCode( int freeReg, int freeTemp ) {
		int arrayWeight = array.weight();
		int subscriptWeight = subscript.weight();
		if ( subscriptWeight < MAXREG - freeReg ) {
			array.regCode( freeReg, freeTemp );
			subscript.regCode( freeReg + 1, freeTemp );
			load( tempReg( freeReg + 2 ), displacement( 0, tempReg( freeReg ) ) );	// Size
			instrn( "blt", tempReg( freeReg + 1 ), "IO.error.enter" );
			instrn( "subq", tempReg( freeReg + 1 ), 
				tempReg( freeReg + 2 ), tempReg( freeReg + 2 ) );
			instrn( "bge", tempReg( freeReg + 2 ), "IO.error.enter" );
			load( tempReg( freeReg + 2 ), displacement( 8, tempReg( freeReg ) ) );	// Array address
			instrn( "s8addq", tempReg( freeReg + 1 ), 
				tempReg( freeReg + 2 ), tempReg( freeReg ) );
			}
		else if ( arrayWeight < MAXREG - freeReg ) {
			subscript.regCode( freeReg, freeTemp );
			array.regCode( freeReg + 1, freeTemp );
			load( tempReg( freeReg + 2 ), displacement( 0, tempReg( freeReg + 1 ) ) );	// Size
			instrn( "blt", tempReg( freeReg ), "IO.error.enter" );
			instrn( "subq", tempReg( freeReg ), 
				tempReg( freeReg + 2 ), tempReg( freeReg + 2 ) );
			instrn( "bge", tempReg( freeReg + 2 ), "IO.error.enter" );
			load( tempReg( freeReg + 2 ), displacement( 8, tempReg( freeReg + 1 ) ) );	// Array address
			instrn( "s8addq", tempReg( freeReg ), 
				tempReg( freeReg + 2 ), tempReg( freeReg ) );
			}
		else if ( subscriptWeight <= arrayWeight ) {
			array.memCode( freeTemp );
			subscript.regCode( freeReg, freeTemp + 1 );
			load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );	//	array
			load( tempReg( freeReg + 2 ), displacement( 0, tempReg( freeReg + 1 ) ) );	// Size
			instrn( "blt", tempReg( freeReg ), "IO.error.enter" );
			instrn( "subq", tempReg( freeReg ), 
				tempReg( freeReg + 2 ), tempReg( freeReg + 2 ) );
			instrn( "bge", tempReg( freeReg + 2 ), "IO.error.enter" );
			load( tempReg( freeReg + 2 ), displacement( 8, tempReg( freeReg + 1 ) ) );	// Array address
			instrn( "s8addq", tempReg( freeReg ), 
				tempReg( freeReg + 2 ), tempReg( freeReg ) );
			}
		else {
			subscript.memCode( freeTemp );
			array.regCode( freeReg, freeTemp + 1 );
			load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );	//	Subscript
			load( tempReg( freeReg + 2 ), displacement( 0, tempReg( freeReg ) ) );	// Size
			instrn( "blt", tempReg( freeReg + 1 ), "IO.error.enter" );
			instrn( "subq", tempReg( freeReg + 1 ), 
				tempReg( freeReg + 2 ), tempReg( freeReg + 2 ) );
			instrn( "bge", tempReg( freeReg + 2 ), "IO.error.enter" );
			load( tempReg( freeReg + 2 ), displacement( 8, tempReg( freeReg ) ) );	// Array address
			instrn( "s8addq", tempReg( freeReg + 1 ), 
				tempReg( freeReg + 2 ), tempReg( freeReg ) );
			}
		return indirect( tempReg( freeReg ) );
		}
	}
