package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class DoStmtNode extends StmtNode {

	private DeclStmtListNode stmtList;
	private ExprNode cond;

	public DoStmtNode( DeclStmtListNode stmtList, ExprNode cond ) {
		this.stmtList = stmtList;
		this.cond = cond;
		}

	public String toString() {
		return "do%+%n" + stmtList + "%-%nend%nwhile " + cond + ";";
		}

	public void genEnv( Env env ) {
		stmtList.genEnv( Decl.SECT_LOCAL, env );
		cond.genEnv( env );
		}

	public void checkType() {
		stmtList.checkType();
		Type condType = cond.checkType();
		cond = cond.castTo( BoolType.type );
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		do
			stmtList.eval( runEnv );
		while ( cond.eval( runEnv ).boolValue() );
		}

	public int calcWeight() {
		int stmtListWeight = stmtList.calcWeight();
		int condWeight = cond.calcWeight();
		weight = max( stmtListWeight, condWeight );
		return weight;
		}

	public void genCode() {
		enter();
		labelDefn( "do" );
		stmtList.genCode();
		labelDefn( "continue" );
		cond.boolCode( 0, 0, "do", "end", false );
		labelDefn( "break" );
		labelDefn( "end" );
		exit();
		}
	}
