package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class WhileStmtNode extends StmtNode {

	private ExprNode cond;
	private DeclStmtListNode stmtList;

	public WhileStmtNode( ExprNode cond, DeclStmtListNode stmtList ) {
		this.cond = cond;
		this.stmtList = stmtList;
		}

	public String toString() {
		return "while " + cond + " do%+%n" + stmtList + "%-%nend";
		}

	public void genEnv( Env env ) {
		cond.genEnv( env );
		stmtList.genEnv( Decl.SECT_LOCAL, env );
		}

	public void checkType() {
		Type condType = cond.checkType();
		cond = cond.castTo( BoolType.type );
		stmtList.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		while ( cond.eval( runEnv ).boolValue() )
			stmtList.eval( runEnv );
		}

	public int calcWeight() {
		int condWeight = cond.calcWeight();
		int stmtListWeight = stmtList.calcWeight();
		weight = max( condWeight, stmtListWeight );
		return weight;
		}

	public void genCode() {
		enter();
		labelDefn( "while" );
		cond.boolCode( 0, 0, "do", "end", true );
		labelDefn( "do" );
		stmtList.genCode();
		labelDefn( "continue" );
		br( "while" );
		labelDefn( "break" );
		labelDefn( "end" );
		exit();
		}
	}
