package node.stmtNode.elseOptNode;
import node.*;
import node.stmtNode.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class ElseOpt2Node extends ElseOptNode {

	private ExprNode cond;
	private DeclStmtListNode thenPart;
	private ElseOptNode elsePart;

	public ElseOpt2Node( 
		ExprNode cond,
		DeclStmtListNode thenPart,
		ElseOptNode elsePart ) {
		this.cond = cond;
		this.thenPart = thenPart;
		this.elsePart = elsePart;
		}

	public String toString() {
		return "%nelif " + cond + " then%+%n" + thenPart + "%-" + elsePart;
		}

	public void genEnv( Env env ) {
		cond.genEnv( env );
		thenPart.genEnv( Decl.SECT_LOCAL, env );
		elsePart.genEnv( env );
		}

	public void checkType() {
		Type condType = cond.checkType();
		cond = cond.castTo( BoolType.type );
		thenPart.checkType();
		elsePart.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		if ( cond.eval( runEnv ).boolValue() )
			thenPart.eval( runEnv );
		else
			elsePart.eval( runEnv );
		}

	public int calcWeight() {
		int condWeight = cond.calcWeight();
		int thenWeight = thenPart.calcWeight();
		int elseWeight = elsePart.calcWeight();
		weight = max( condWeight, thenWeight, elseWeight );
		return weight;
		}

	public void genCode() {
		enter();
		labelDefn( "if" );
		cond.boolCode( 0, 0, "then", "else", true );
		labelDefn( "then" );
		thenPart.genCode();
		br( "end" );
		labelDefn( "else" );
		elsePart.genCode();
		labelDefn( "end" );
		exit();
		}
	}
