package grammar;

import java.io.*;

%%

%{
	int commentNest = 0;
	int braceNest = 0;
	int sectionCount = 0;

	int lineCount = 1;
	
	boolean printed = false;
	
	void echo( String text ) {
		System.out.print( text );
		printed = true;
		}
%}

%init{
	yybegin( NORMAL );
%init}

%public
%type Void

%state NORMAL COMMENT CODESTRING

newline		=	\r|\n|\r\n
%%

<NORMAL> {
	"%%"			{
					sectionCount++;
					if ( sectionCount == 2 )
						return null;
					echo( yytext() );
					}
	"/*"			{
					yybegin( COMMENT );
					commentNest++;
					}
	"{"				{ 
					yybegin( CODESTRING ); 
					braceNest++;
					}
	{newline}		{
					if ( printed ) {
						System.out.println();
						printed = false;
						}
					lineCount++;
					}
	\'[^\']\'		{ echo( yytext() ); }
	.				{ echo( yytext() ); }
	}
	
<COMMENT> {
	"/*"			{ commentNest++; }
	"*/"			{
					--commentNest;
					if ( commentNest == 0 )
						yybegin( NORMAL );
					}
	{newline}		{ lineCount++; }
	.				{ }
	}
	
<CODESTRING> {
	"{"				{ braceNest++; }
	"}"				{ 
					--braceNest;
					if ( braceNest == 0 )
						yybegin( NORMAL );
					}
	{newline}		{ lineCount++; }
	\'[^\']\'		{ }
	.				{ }
	}
